from django.core.signals import setting_changed
from django.dispatch import receiver

from .finders import get_finders


@receiver(setting_changed)
def clear_embed_caches(*, setting: str, **kwargs: dict) -> None:
    """
    Clear the embed caches when settings change
    """
    if setting == "WAGTAILEMBEDS_FINDERS":
        get_finders.cache_clear()



from django.core.exceptions import ImproperlyConfigured
from django.forms import Media, MediaDefiningClass
from django.utils.functional import cached_property

from wagtail import hooks
from wagtail.admin.ui.sidebar import LinkMenuItem as LinkMenuItemComponent
from wagtail.admin.ui.sidebar import SubMenuItem as SubMenuItemComponent
from wagtail.coreutils import cautious_slugify


class MenuItem(metaclass=MediaDefiningClass):
    def __init__(
        self,
        label,
        url,
        name=None,
        classname="",
        icon_name="",
        attrs=None,
        order=1000,
    ):
        self.label = label
        self.url = url
        self.classname = classname
        self.icon_name = icon_name
        self.name = name or cautious_slugify(str(label))
        self.attrs = attrs or {}
        self.order = order

    def is_shown(self, request):
        """
        Whether this menu item should be shown for the given request; permission
        checks etc should go here. By default, menu items are shown all the time
        """
        return True

    def render_component(self, request):
        return LinkMenuItemComponent(
            self.name,
            self.label,
            self.url,
            icon_name=self.icon_name,
            classname=self.classname,
            attrs=self.attrs,
        )


class DismissibleMenuItemMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.attrs["data-controller"] = "w-dismissible"
        self.attrs["data-w-dismissible-dismissed-class"] = "w-dismissible--dismissed"
        self.attrs["data-w-dismissible-id-value"] = self.name

    def render_component(self, request):
        profile = getattr(request.user, "wagtail_userprofile", None)

        # Menu item instances are cached, so make sure the existence of the
        # data-w-dismissible-dismissed-value attribute is correct for the user
        if profile and profile.dismissibles.get(self.name):
            self.attrs["data-w-dismissible-dismissed-value"] = "true"
        else:
            self.attrs.pop("data-w-dismissible-dismissed-value", None)

        return super().render_component(request)


class DismissibleMenuItem(DismissibleMenuItemMixin, MenuItem):
    pass


class Menu:
    def __init__(self, register_hook_name=None, construct_hook_name=None, items=None):
        if register_hook_name is not None and not isinstance(register_hook_name, str):
            raise ImproperlyConfigured(
                "Expected a string or None as register_hook_name, got %r. "
                "Did you mean to pass an `items` keyword argument instead?"
                % register_hook_name
            )

        self.register_hook_name = register_hook_name
        self.construct_hook_name = construct_hook_name
        self.initial_menu_items = items

    @cached_property
    def registered_menu_items(self):
        # Construct the list of menu items from the set passed to the constructor along with any
        # registered through hooks. We can't do this in __init__ because we can't rely on all hooks
        # modules to have been imported at the point that we create the admin_menu and
        # settings_menu instances
        if self.initial_menu_items:
            items = self.initial_menu_items.copy()
        else:
            items = []

        if self.register_hook_name:
            for fn in hooks.get_hooks(self.register_hook_name):
                items.append(fn())

        return items

    def menu_items_for_request(self, request):
        items = [item for item in self.registered_menu_items if item.is_shown(request)]

        # provide a hook for modifying the menu, if construct_hook_name has been set
        if self.construct_hook_name:
            for fn in hooks.get_hooks(self.construct_hook_name):
                fn(request, items)

        return items

    @property
    def media(self):
        media = Media()
        for item in self.registered_menu_items:
            media += item.media
        return media

    def render_component(self, request):
        menu_items = self.menu_items_for_request(request)
        rendered_menu_items = []
        for item in sorted(menu_items, key=lambda i: i.order):
            rendered_menu_items.append(item.render_component(request))
        return rendered_menu_items


class SubmenuMenuItem(MenuItem):
    """A MenuItem which wraps an inner Menu object"""

    def __init__(self, label, menu, **kwargs):
        self.menu = menu
        super().__init__(label, "#", **kwargs)

    def is_shown(self, request):
        # show the submenu if one or more of its children is shown
        return bool(self.menu.menu_items_for_request(request))

    def render_component(self, request):
        return SubMenuItemComponent(
            self.name,
            self.label,
            self.menu.render_component(request),
            icon_name=self.icon_name,
            classname=self.classname,
            attrs=self.attrs,
        )


class DismissibleSubmenuMenuItem(DismissibleMenuItemMixin, SubmenuMenuItem):
    pass


class AdminOnlyMenuItem(MenuItem):
    """A MenuItem which is only shown to superusers"""

    def is_shown(self, request):
        return request.user.is_superuser


class WagtailMenuRegisterable:
    #: The icon used for the menu item that appears in Wagtail's sidebar.
    menu_icon = ""

    #: The displayed label used for the menu item.
    menu_label = ""

    #: The ``name`` argument passed to the ``MenuItem`` constructor, becoming the ``name`` attribute value for that instance.
    #: This can be useful when manipulating the menu items in a custom menu hook, e.g. :ref:`construct_main_menu`.
    #: If unset, a slugified version of :attr:`menu_label` is used.
    menu_name = ""

    #: An integer determining the order of the menu item, 0 being the first place.
    #: By default, it will be the last item before Reports, whose order is 9000.
    menu_order = 8999

    #: The URL to be used for the menu item.
    menu_url = None

    #: Register the menu item within the admin's main menu.
    add_to_admin_menu = False

    #: Register the menu item within the admin's "Settings" menu.
    #: This takes precedence if both ``add_to_admin_menu`` and ``add_to_settings_menu`` are set to ``True``.
    add_to_settings_menu = False

    @cached_property
    def menu_item_class(self):
        """A ``wagtail.admin.menu.MenuItem`` subclass to be registered with a menu hook."""
        return type(
            f"{self.__class__.__name__}MenuItem",
            (MenuItem,),
            {},
        )

    def get_menu_item(self, order=None):
        """
        Returns a ``wagtail.admin.menu.MenuItem`` instance to be registered
        with the Wagtail admin.

        The ``order`` parameter allows the method to be called from the outside
        (e.g. a ``ViewSetGroup``) to create a sub menu item with
        the correct order.
        """
        return self.menu_item_class(
            label=self.menu_label,
            url=self.menu_url,
            name=self.menu_name,
            icon_name=self.menu_icon,
            order=order if order is not None else self.menu_order,
        )

    @cached_property
    def menu_hook(self):
        """
        The name of the hook to register the menu item within.

        This takes precedence over ``add_to_admin_menu`` and ``add_to_settings_menu``.
        """
        if self.add_to_settings_menu:
            return "register_settings_menu_item"
        if self.add_to_admin_menu:
            return "register_admin_menu_item"

    def register_menu_item(self):
        """Registers the menu item with the Wagtail admin."""
        if self.menu_hook:
            hooks.register(self.menu_hook, self.get_menu_item)


class WagtailMenuRegisterableGroup(WagtailMenuRegisterable):
    """
    A container for grouping together multiple WagtailMenuRegisterable instances.
    Creates a menu item with a submenu for accessing the main URL for each instances.
    """

    #: A list or tuple of ``WagtailMenuRegisterable`` classes or instances
    #: to be grouped together.
    items = ()

    menu_icon = "folder-open-inverse"
    add_to_admin_menu = True

    def __init__(self):
        """
        When initialising, instantiate the classes (or use the instances)
        within 'items', and assign the list to a ``registerables`` attribute.
        """
        self.registerables = [
            registerable() if callable(registerable) else registerable
            for registerable in self.items
        ]

    def get_submenu_items(self):
        menu_items = []
        item_order = 1
        for registerable in self.registerables:
            menu_items.append(registerable.get_menu_item(order=item_order))
            item_order += 1
        return menu_items

    def get_menu_item(self, order=None):
        return SubmenuMenuItem(
            label=self.menu_label,
            menu=Menu(items=self.get_submenu_items()),
            name=self.menu_name,
            icon_name=self.menu_icon,
            order=order if order is not None else self.menu_order,
        )


admin_menu = Menu(
    register_hook_name="register_admin_menu_item",
    construct_hook_name="construct_main_menu",
)
settings_menu = Menu(
    register_hook_name="register_settings_menu_item",
    construct_hook_name="construct_settings_menu",
)
reports_menu = Menu(
    register_hook_name="register_reports_menu_item",
    construct_hook_name="construct_reports_menu",
)
help_menu = Menu(
    register_hook_name="register_help_menu_item",
    construct_hook_name="construct_help_menu",
)



import re
from typing import Any, Union

from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models.expressions import CombinedExpression, Expression, Value
from django.db.models.fields import BooleanField, Field
from django.db.models.sql.compiler import SQLCompiler


class LexemeCombinable(Expression):
    BITAND = "+"
    BITOR = ""
    invert = False  # By default, it is not inverted

    def _combine(self, other, connector, reversed, node=None):
        if not isinstance(other, LexemeCombinable):
            raise TypeError(
                f"Lexeme can only be combined with other Lexemes, got {type(other)}."
            )
        if reversed:
            return CombinedLexeme(other, connector, self)
        return CombinedLexeme(self, connector, other)

    # On Combinable, these are not implemented to reduce confusion with Q. In
    # this case we are actually (ab)using them to do logical combination so
    # it's consistent with other usage in Django.
    def bitand(self, other):
        return self._combine(other, self.BITAND, False)

    def bitor(self, other):
        return self._combine(other, self.BITOR, False)

    def __or__(self, other):
        return self._combine(other, self.BITOR, False)

    def __and__(self, other):
        return self._combine(other, self.BITAND, False)


class SearchQueryField(Field):
    def db_type(self, connection):
        return None


class Lexeme(LexemeCombinable, Value):
    _output_field = SearchQueryField()

    def __init__(
        self, value, output_field=None, invert=False, prefix=False, weight=None
    ):
        self.prefix = prefix
        self.invert = invert
        self.weight = weight

        if not value:
            raise ValueError("Lexeme value cannot be empty.")
        if re.search(r"\W+", value):
            raise ValueError(
                f"Lexeme value '{value}' must consist of alphanumeric characters and '_' only."
            )
        super().__init__(value, output_field=output_field)

    def as_sql(self, compiler, connection):
        param = self.value
        template = "%s"

        if self.prefix:
            param = f"{param}*"
        if self.invert:
            param = f"(-{param})"
        else:
            param = f"{param}"

        return template, [param]


class CombinedLexeme(LexemeCombinable):
    _output_field = SearchQueryField()

    def __init__(self, lhs, connector, rhs, output_field=None):
        super().__init__(output_field=output_field)
        self.connector = connector
        self.lhs = lhs
        self.rhs = rhs

    def as_sql(self, compiler, connection):
        value_params = []

        lhs_connector = self.connector
        rhs_connector = self.connector

        if (
            self.lhs.invert and self.connector == "+"
        ):  # NOTE: This is a special case for MySQL. If either side's operator is AND (+), and it is inverted, the operator should become NOT (-). If we did nothing, the result could would be '+X +(-Y)' for And(X, Not(Y)), which seems correct, but produces a wrong result. The solution is to turn the query into '+X -Y', which does work, and therefore this is done here.
            # TODO: There may be a better solution than this.
            modified_value = self.lhs.value.copy()
            modified_value.invert = not modified_value.invert
            lhs_connector = "-"
            lsql, params = compiler.compile(modified_value)
        else:
            lsql, params = compiler.compile(self.lhs)
        value_params.extend(params)

        if self.rhs.invert and self.connector == "+":  # Same explanation as above.
            modified_value = self.rhs.value.copy()
            modified_value.invert = not modified_value.invert
            rhs_connector = "-"
            rsql, params = compiler.compile(modified_value)
        else:
            rsql, params = compiler.compile(self.rhs)
        value_params.extend(params)

        combined_sql = "({}{} {}{})".format(
            lhs_connector, lsql, rhs_connector, rsql
        )  # if self.connector is '+' (AND), then both terms will be ANDed together. We need to repeat the connector to make that work.
        combined_value = combined_sql % tuple(value_params)
        return "%s", [combined_value]


class SearchQueryCombinable:
    BITAND = "+"
    BITOR = ""

    def _combine(self, other, connector: str, reversed: bool = False):
        if not isinstance(other, SearchQueryCombinable):
            raise TypeError(
                "SearchQuery can only be combined with other SearchQuery "
                "instances, got %s." % type(other).__name__
            )
        if reversed:
            return CombinedSearchQuery(other, connector, self)
        return CombinedSearchQuery(self, connector, other)

    # On Combinable, these are not implemented to reduce confusion with Q. In
    # this case we are actually (ab)using them to do logical combination so
    # it's consistent with other usage in Django.
    def __or__(self, other):
        return self._combine(other, self.BITOR, False)

    def __ror__(self, other):
        return self._combine(other, self.BITOR, True)

    def __and__(self, other):
        return self._combine(other, self.BITAND, False)

    def __rand__(self, other):
        return self._combine(other, self.BITAND, True)


class SearchQuery(SearchQueryCombinable, Expression):
    def __init__(
        self, value: Union[LexemeCombinable, str], search_type: str = "lexeme", **extra
    ):
        super().__init__(output_field=SearchQueryField())
        self.extra = extra
        if (
            isinstance(value, str) or search_type == "phrase"
        ):  # If the value is a string, we assume it's a phrase
            safe_string = re.sub(
                r"\W+", " ", value
            )  # Remove non-word characters. This is done to disallow the usage of full text search operators in the MATCH clause, because MySQL doesn't include these kinds of characters in FULLTEXT indexes.
            self.value = Value(
                '"%s"' % safe_string
            )  # We wrap it in quotes to make sure it's parsed as a phrase
        else:  # Otherwise, we assume it's a lexeme
            self.value = value

    def as_sql(
        self,
        compiler: SQLCompiler,
        connection: BaseDatabaseWrapper,
        **extra_context: Any,
    ) -> tuple[str, list[Any]]:
        sql, params = compiler.compile(self.value)
        return (sql, params)

    def __repr__(self) -> str:
        return self.value.__repr__()


class CombinedSearchQuery(SearchQueryCombinable, CombinedExpression):
    def __init__(self, lhs, connector, rhs, output_field=None):
        super().__init__(lhs, connector, rhs, output_field)

    def __str__(self):
        return "%s" % super().__str__()

    def as_sql(self, compiler, connection):
        value_params = []

        lhs_connector = self.connector
        rhs_connector = self.connector

        if (
            isinstance(self.lhs, SearchQuery)
            and isinstance(self.lhs.value, Lexeme)
            and self.lhs.value.invert
            and self.connector == "+"
        ):  # NOTE: The explanation for this special case is the same as above, in the CombinedLexeme class.
            modified_value = self.lhs.value.copy()
            modified_value.invert = not modified_value.invert
            lhs_connector = "-"
            lsql, params = compiler.compile(modified_value)
        else:
            lsql, params = compiler.compile(self.lhs)
        value_params.extend(params)

        if (
            isinstance(self.rhs, SearchQuery)
            and isinstance(self.rhs.value, Lexeme)
            and self.rhs.value.invert
            and self.connector == "+"
        ):  # NOTE: The explanation for this special case is the same as above, in the CombinedLexeme class.
            modified_value = self.rhs.value.copy()
            modified_value.invert = not modified_value.invert
            rhs_connector = "-"
            rsql, params = compiler.compile(modified_value)
        else:
            rsql, params = compiler.compile(self.rhs)
        value_params.extend(params)

        combined_sql = "({}{} {}{})".format(
            lhs_connector, lsql, rhs_connector, rsql
        )  # if self.connector is '+' (AND), then both terms will be ANDed together. We need to repeat the connector to make that work.
        combined_value = combined_sql % tuple(value_params)
        return "%s", [combined_value]


class MatchExpression(Expression):
    filterable = True
    template = "MATCH (%s) AGAINST (%s IN BOOLEAN MODE)"

    def __init__(
        self,
        query: SearchQueryCombinable,
        columns: list[str] = None,
        output_field: Field = BooleanField(),
    ) -> None:
        super().__init__(output_field=output_field)
        self.query = query
        self.columns = (
            columns
            or [
                "title",
                "body",
            ]
        )  # We need to provide a default list of columns if the user doesn't specify one. We have a joint index for for 'title' and 'body' (see wagtail.search.migrations.0006_customise_indexentry), so we'll pick that one.

    def as_sql(self, compiler, connection):
        compiled_query = compiler.compile(self.query)  # Compile the query to a string
        formatted_query = compiled_query[0] % tuple(
            compiled_query[1]
        )  # Substitute the params in the query
        column_list = ", ".join(
            [f"`{column}`" for column in self.columns]
        )  # ['title', 'body'] becomes '`title`, `body`'
        params = [formatted_query]
        return (self.template % (column_list, "%s"), params)



import datetime
import json

from django.db import connection
from django.db.models import F, JSONField, TextField
from django.db.models.functions import Cast
from django.test import TestCase
from django.utils import timezone

from wagtail.blocks.migrations.operations import (
    RenameStreamChildrenOperation,
    StreamChildrenToListBlockOperation,
)
from wagtail.test.streamfield_migrations import factories, models
from wagtail.test.streamfield_migrations.testutils import MigrationTestMixin

# TODO test multiple operations in one go


class BaseMigrationTest(TestCase, MigrationTestMixin):
    factory = None
    has_revisions = False
    default_operation_and_block_path = [
        (
            RenameStreamChildrenOperation(old_name="char1", new_name="renamed1"),
            "",
        )
    ]
    app_name = None

    def _get_test_instances(self):
        return [
            self.factory(
                content__0__char1="Test char 1",
                content__1__char1="Test char 2",
                content__2__char2="Test char 3",
                content__3__char2="Test char 4",
            ),
            self.factory(
                content__0__char1="Test char 1",
                content__1__char1="Test char 2",
                content__2__char2="Test char 3",
            ),
            self.factory(
                content__0__char2="Test char 1",
                content__1__char2="Test char 2",
                content__2__char2="Test char 3",
            ),
        ]

    def setUp(self):
        instances = self._get_test_instances()

        self.original_raw_data = {}
        self.original_revisions = {}

        for instance in instances:
            self.original_raw_data[instance.id] = instance.content.raw_data

            if self.has_revisions:
                for i in range(5):
                    revision = instance.save_revision()
                    revision.created_at = timezone.now() - datetime.timedelta(
                        days=(5 - i)
                    )
                    revision.save()
                    if i == 1:
                        instance.live_revision = revision
                        instance.save()
                self.original_revisions[instance.id] = list(
                    instance.revisions.all().order_by("id")
                )

    def assertBlocksRenamed(self, old_content, new_content, is_altered=True):
        for old_block, new_block in zip(old_content, new_content):
            self.assertEqual(old_block["id"], new_block["id"])
            if is_altered and old_block["type"] == "char1":
                self.assertEqual(new_block["type"], "renamed1")
            else:
                self.assertEqual(old_block["type"], new_block["type"])

    def _test_migrate_stream_data(self):
        """Test whether the stream data of the model instances have been updated properly

        Apply the migration and then query the raw data of the updated instances. Compare with
        original raw data and check whether all relevant `char1` blocks have been renamed and
        whether ids and other block types are intact.
        """

        self.apply_migration()

        instances = self.model.objects.all().annotate(
            raw_content=Cast(F("content"), JSONField())
        )

        for instance in instances:
            prev_content = self.original_raw_data[instance.id]
            self.assertBlocksRenamed(
                old_content=prev_content, new_content=instance.raw_content
            )

    # TODO test multiple operations applied in one migration

    def _test_migrate_revisions(self):
        """Test whether all revisions have been updated properly

        Applying migration with `revisions_from=None`, so all revisions should be updated.
        """

        self.apply_migration()

        instances = self.model.objects.all()

        for instance in instances:
            old_revisions = self.original_revisions[instance.id]
            for old_revision, new_revision in zip(
                old_revisions, instance.revisions.all().order_by("id")
            ):
                old_content = json.loads(old_revision.content["content"])
                new_content = json.loads(new_revision.content["content"])
                self.assertBlocksRenamed(
                    old_content=old_content, new_content=new_content
                )

    def _test_always_migrate_live_and_latest_revisions(self):
        """Test whether latest and live revisions are always updated

        Applying migration with `revisions_from` set to a date in the future, so there should be
        no revisions which are made after the date. Only the live and latest revisions should
        update in this case.
        """

        revisions_from = timezone.now() + datetime.timedelta(days=2)
        self.apply_migration(revisions_from=revisions_from)

        instances = self.model.objects.all()

        for instance in instances:
            old_revisions = self.original_revisions[instance.id]
            for old_revision, new_revision in zip(
                old_revisions, instance.revisions.all().order_by("id")
            ):
                is_latest_or_live = old_revision.id == instance.live_revision_id or (
                    old_revision.id == instance.latest_revision_id
                )
                old_content = json.loads(old_revision.content["content"])
                new_content = json.loads(new_revision.content["content"])
                self.assertBlocksRenamed(
                    old_content=old_content,
                    new_content=new_content,
                    is_altered=is_latest_or_live,
                )

    def _test_migrate_revisions_from_date(self):
        """Test whether revisions from a given date onwards are updated

        Applying migration with `revisions_from` set to a date between the created date of the first
        and last revision, so only the revisions after the date and the live and latest revision
        should be updated.
        """

        revisions_from = timezone.now() - datetime.timedelta(days=2)
        self.apply_migration(revisions_from=revisions_from)

        instances = self.model.objects.all()

        for instance in instances:
            old_revisions = self.original_revisions[instance.id]
            for old_revision, new_revision in zip(
                old_revisions, instance.revisions.all().order_by("id")
            ):
                is_latest_or_live = old_revision.id == instance.live_revision_id or (
                    old_revision.id == instance.latest_revision_id
                )
                is_after_revisions_from = old_revision.created_at > revisions_from
                is_altered = is_latest_or_live or is_after_revisions_from
                old_content = json.loads(old_revision.content["content"])
                new_content = json.loads(new_revision.content["content"])
                self.assertBlocksRenamed(
                    old_content=old_content,
                    new_content=new_content,
                    is_altered=is_altered,
                )


class TestNonPageModelWithoutRevisions(BaseMigrationTest):
    model = models.SampleModel
    factory = factories.SampleModelFactory
    has_revisions = False
    app_name = "streamfield_migration_tests"

    def test_migrate_stream_data(self):
        self._test_migrate_stream_data()


class TestPage(BaseMigrationTest):
    model = models.SamplePage
    factory = factories.SamplePageFactory
    has_revisions = True
    app_name = "streamfield_migration_tests"

    def test_migrate_stream_data(self):
        self._test_migrate_stream_data()

    def test_migrate_revisions(self):
        self._test_migrate_revisions()

    def test_always_migrate_live_and_latest_revisions(self):
        self._test_always_migrate_live_and_latest_revisions()

    def test_migrate_revisions_from_date(self):
        self._test_migrate_revisions_from_date()


class TestNullStreamField(BaseMigrationTest):
    """
    Migrations are processed if the underlying JSON is null.

    This might occur if we're operating on a StreamField that was added to a model that
    had existing records.
    """

    model = models.SamplePage
    factory = factories.SamplePageFactory
    has_revisions = True
    app_name = "streamfield_migration_tests"

    def _get_test_instances(self):
        return self.factory.create_batch(1, content=None)

    def setUp(self):
        super().setUp()

        # Bypass StreamField/StreamBlock processing that cast a None stream field value
        # to the empty StreamValue, and set the underlying JSON to null.
        with connection.cursor() as cursor:
            cursor.execute(f"UPDATE {self.model._meta.db_table} SET content = 'null'")

    def assert_null_content(self):
        """
        The raw JSON of all instances for this test is null.
        """

        instances = self.model.objects.all().annotate(
            raw_content=Cast(F("content"), TextField())
        )

        for instance in instances:
            with self.subTest(instance=instance):
                self.assertEqual(instance.raw_content, "null")

    def test_migrate_stream_data(self):
        self.assert_null_content()
        self.apply_migration()
        self.assert_null_content()


class StreamChildrenToListBlockOperationTestCase(BaseMigrationTest):
    model = models.SamplePage
    factory = factories.SamplePageFactory
    has_revisions = True
    app_name = "streamfield_migration_tests"

    def _get_test_instances(self):
        return self.factory.create_batch(
            size=3,
            # Each content stream field has a single char block instance.
            content__0__char1__value="Char Block 1",
        )

    def test_state_not_shared_across_instances(self):
        """
        StreamChildrenToListBlockOperation doesn't share state across model instances.

        As a single operation instance is used to transform the data of multiple model
        instances, we should not store model instance state on the operation instance.
        See https://github.com/wagtail/wagtail/issues/12391.
        """

        self.apply_migration(
            operations_and_block_paths=[
                (
                    StreamChildrenToListBlockOperation(
                        block_name="char1", list_block_name="list1"
                    ),
                    "",
                )
            ]
        )
        for instance in self.model.objects.all().annotate(
            raw_content=Cast(F("content"), JSONField())
        ):
            new_block = instance.raw_content[0]
            self.assertEqual(new_block["type"], "list1")
            self.assertEqual(len(new_block["value"]), 1)
            self.assertEqual(new_block["value"][0]["type"], "item")
            self.assertEqual(new_block["value"][0]["value"], "Char Block 1")



# Generated by Django 4.0.4 on 2022-05-25 15:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0072_alter_revision_content_type_notnull"),
    ]

    operations = [
        migrations.AddField(
            model_name="page",
            name="latest_revision",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailcore.revision",
                verbose_name="latest revision",
            ),
        ),
    ]



import unittest.mock

from django.apps import apps
from django.template import TemplateSyntaxError, engines
from django.test import TestCase

from wagtail.models import Site

from .utils import (
    Image,
    get_test_bad_image,
    get_test_image_file,
    get_test_image_filename,
)


class JinjaImagesTestCase(TestCase):
    maxDiff = None

    def setUp(self):
        self.engine = engines["jinja2"]

        self.image = Image.objects.create(
            title="Test image",
            file=get_test_image_file(),
        )

        self.bad_image = get_test_bad_image()
        self.bad_image.save()

    def render(self, string, context=None, request_context=True):
        if context is None:
            context = {}

        # Add a request to the template, to simulate a RequestContext
        if request_context:
            site = Site.objects.get(is_default_site=True)
            request = self.client.get("/test/", HTTP_HOST=site.hostname)
            context["request"] = request

        template = self.engine.from_string(string)
        return template.render(context)


class TestImageJinja(JinjaImagesTestCase):
    def test_image(self):
        self.assertHTMLEqual(
            self.render('{{ image(myimage, "width-200") }}', {"myimage": self.image}),
            '<img alt="Test image" src="{}" width="200" height="150">'.format(
                get_test_image_filename(self.image, "width-200")
            ),
        )

    def test_no_image(self):
        rendered = self.render('{{ image(myimage, "width-2") }}', {"myimage": None})
        self.assertEqual(rendered, "")

    def test_image_attributes(self):
        self.assertHTMLEqual(
            self.render(
                '{{ image(myimage, "width-200", alt="alternate", class="test") }}',
                {"myimage": self.image},
            ),
            '<img alt="alternate" src="{}" width="200" height="150" class="test">'.format(
                get_test_image_filename(self.image, "width-200")
            ),
        )

    def test_image_assignment(self):
        template = (
            '{% set bg=image(myimage, "width-200") %}'
            "width: {{ bg.width }}, url: {{ bg.url }}"
        )
        output = "width: 200, url: " + get_test_image_filename(self.image, "width-200")
        self.assertHTMLEqual(self.render(template, {"myimage": self.image}), output)

    def test_image_assignment_render_as_is(self):
        self.assertHTMLEqual(
            self.render(
                '{% set bg=image(myimage, "width-200") %}{{ bg }}',
                {"myimage": self.image},
            ),
            '<img alt="Test image" src="{}" width="200" height="150">'.format(
                get_test_image_filename(self.image, "width-200")
            ),
        )

    def test_missing_image(self):
        self.assertHTMLEqual(
            self.render(
                '{{ image(myimage, "width-200") }}', {"myimage": self.bad_image}
            ),
            '<img alt="missing image" src="/media/not-found" width="0" height="0">',
        )

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in 'image' tag may only"
        ):
            self.render('{{ image(myimage, "fill-200×200") }}', {"myimage": self.image})

    def test_custom_default_attrs(self):
        with unittest.mock.patch.object(
            apps.get_app_config("wagtailimages"),
            "default_attrs",
            new={"decoding": "async", "loading": "lazy"},
        ):
            self.assertHTMLEqual(
                self.render(
                    '{{ image(myimage, "width-200") }}', {"myimage": self.bad_image}
                ),
                '<img alt="missing image" src="/media/not-found" width="0" height="0" decoding="async" loading="lazy">',
            )

    def test_chaining_filterspecs(self):
        self.assertHTMLEqual(
            self.render(
                '{{ image(myimage, "width-200|jpegquality-40") }}',
                {"myimage": self.image},
            ),
            '<img alt="Test image" src="{}" width="200" height="150">'.format(
                get_test_image_filename(self.image, "width-200.jpegquality-40")
            ),
        )


class TestImageURLJinja(JinjaImagesTestCase):
    def test_image_url(self):
        self.assertRegex(
            self.render(
                '{{ image_url(myimage, "width-200") }}', {"myimage": self.image}
            ),
            "/images/.*/width-200/{}".format(self.image.file.name.split("/")[-1]),
        )

    def test_image_url_custom_view(self):
        self.assertRegex(
            self.render(
                '{{ image_url(myimage, "width-200", "wagtailimages_serve_custom_view") }}',
                {"myimage": self.image},
            ),
            "/testimages/custom_view/.*/width-200/{}".format(
                self.image.file.name.split("/")[-1]
            ),
        )


class TestSrcsetImageJinja(JinjaImagesTestCase):
    def test_srcset_image(self):
        filename_200 = get_test_image_filename(self.image, "width-200")
        filename_400 = get_test_image_filename(self.image, "width-400")
        rendered = self.render(
            '{{ srcset_image(myimage, "width-{200,400}", sizes="100vw") }}',
            {"myimage": self.image},
        )
        expected = f"""
            <img
                sizes="100vw"
                src="{filename_200}"
                srcset="{filename_200} 200w, {filename_400} 400w"
                alt="Test image"
                width="200"
                height="150"
            >
        """
        self.assertHTMLEqual(rendered, expected)

    def test_no_image(self):
        rendered = self.render(
            '{{ srcset_image(myimage, "width-2") }}', {"myimage": None}
        )
        self.assertEqual(rendered, "")

    def test_srcset_output_single_image(self):
        self.assertHTMLEqual(
            self.render(
                '{{ srcset_image(myimage, "width-200") }}',
                {"myimage": self.image},
            ),
            self.render(
                '{{ image(myimage, "width-200") }}',
                {"myimage": self.image},
            ),
        )

    def test_srcset_image_assignment(self):
        template = (
            '{% set bg=srcset_image(myimage, "width-{200,400}") %}'
            "width: {{ bg.renditions[0].width }}, url: {{ bg.renditions[0].url }} "
            "width: {{ bg.renditions[1].width }}, url: {{ bg.renditions[1].url }} "
        )
        rendered = self.render(template, {"myimage": self.image})
        expected = f"""
            width: 200, url: {get_test_image_filename(self.image, "width-200")}
            width: 400, url: {get_test_image_filename(self.image, "width-400")}
        """
        self.assertHTMLEqual(rendered, expected)

    def test_srcset_image_assignment_render_as_is(self):
        rendered = self.render(
            '{% set bg=srcset_image(myimage, "width-{200,400}") %}{{ bg }}',
            {"myimage": self.image},
        )
        expected = self.render(
            '{{ srcset_image(myimage, "width-{200,400}") }}',
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_missing_srcset_image(self):
        rendered = self.render(
            '{{ srcset_image(myimage, "width-{200,400}", sizes="100vw") }}',
            {"myimage": self.bad_image},
        )
        expected = """
            <img
                sizes="100vw"
                src="/media/not-found"
                srcset="/media/not-found 0w, /media/not-found 0w"
                alt="missing image"
                width="0"
                height="0"
            >
        """
        self.assertHTMLEqual(rendered, expected)

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in 'srcset_image' tag may only"
        ):
            self.render(
                '{{ srcset_image(myimage, "fill-{20×20,40×40}", sizes="100vw") }}',
                {"myimage": self.image},
            )

    def test_custom_default_attrs(self):
        with unittest.mock.patch.object(
            apps.get_app_config("wagtailimages"),
            "default_attrs",
            new={"decoding": "async", "loading": "lazy"},
        ):
            rendered = self.render(
                '{{ srcset_image(myimage, "width-{20,40}", sizes="100vw") }}',
                {"myimage": self.bad_image},
            )
            expected = """
                <img
                    sizes="100vw"
                    src="/media/not-found"
                    srcset="/media/not-found 0w, /media/not-found 0w"
                    alt="missing image"
                    width="0"
                    height="0"
                    decoding="async"
                    loading="lazy"
                >
            """
            self.assertHTMLEqual(rendered, expected)

    def test_chaining_filterspecs(self):
        filenames = [
            get_test_image_filename(self.image, "width-200.jpegquality-40"),
            get_test_image_filename(self.image, "width-400.jpegquality-40"),
        ]
        rendered = self.render(
            '{{ srcset_image(myimage, "width-{200,400}|jpegquality-40", sizes="100vw") }}',
            {"myimage": self.image},
        )
        expected = f"""
            <img
                sizes="100vw"
                src="{filenames[0]}"
                srcset="{filenames[0]} 200w, {filenames[1]} 400w"
                alt="Test image"
                width="200"
                height="150"
            >
        """
        self.assertHTMLEqual(rendered, expected)


class TestPictureJinja(JinjaImagesTestCase):
    def test_picture_formats_multi_sizes(self):
        filenames = [
            get_test_image_filename(self.image, "width-200.format-jpeg"),
            get_test_image_filename(self.image, "width-400.format-jpeg"),
            get_test_image_filename(self.image, "width-200.format-webp"),
            get_test_image_filename(self.image, "width-400.format-webp"),
            get_test_image_filename(self.image, "width-200.format-gif"),
            get_test_image_filename(self.image, "width-400.format-gif"),
        ]

        rendered = self.render(
            '{{ picture(myimage, "width-{200,400}|format-{jpeg,webp,gif}", sizes="100vw") }}',
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filenames[2]} 200w, {filenames[3]} 400w" sizes="100vw" type="image/webp">
            <source srcset="{filenames[0]} 200w, {filenames[1]} 400w" sizes="100vw" type="image/jpeg">
            <img
                sizes="100vw"
                src="{filenames[4]}"
                srcset="{filenames[4]} 200w, {filenames[5]} 400w"
                alt="Test image"
                width="200"
                height="150"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_formats_only(self):
        filename_jpeg = get_test_image_filename(self.image, "format-jpeg")
        filename_webp = get_test_image_filename(self.image, "format-webp")

        rendered = self.render(
            '{{ picture(myimage, "format-{jpeg,webp}") }}',
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filename_webp}" type="image/webp">
            <img
                src="{filename_jpeg}"
                alt="Test image"
                width="640"
                height="480"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_sizes_only(self):
        rendered = self.render(
            '{{ picture(myimage, "width-{200,400}", sizes="100vw") }}',
            {"myimage": self.image},
        )
        expected = self.render(
            '<picture>{{ srcset_image(myimage, "width-{200,400}", sizes="100vw") }}</picture>',
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_picture_single_format(self):
        rendered = self.render(
            '{{ picture(myimage, "format-jpeg") }}',
            {"myimage": self.image},
        )
        expected = self.render(
            '<picture>{{ image(myimage, "format-jpeg") }}</picture>',
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_no_image(self):
        rendered = self.render('{{ picture(myimage, "width-2") }}', {"myimage": None})
        self.assertEqual(rendered, "")

    def test_picture_assignment(self):
        template = (
            '{% set bg=picture(myimage, "width-{200,400}|format-{jpeg,webp}") %}'
            "width: {{ bg.formats['jpeg'][0].width }}, url: {{ bg.formats['jpeg'][0].url }} "
            "width: {{ bg.formats['jpeg'][1].width }}, url: {{ bg.formats['jpeg'][1].url }} "
            "width: {{ bg.formats['webp'][0].width }}, url: {{ bg.formats['webp'][0].url }} "
            "width: {{ bg.formats['webp'][1].width }}, url: {{ bg.formats['webp'][1].url }} "
        )
        rendered = self.render(template, {"myimage": self.image})
        expected = f"""
            width: 200, url: {get_test_image_filename(self.image, "width-200.format-jpeg")}
            width: 400, url: {get_test_image_filename(self.image, "width-400.format-jpeg")}
            width: 200, url: {get_test_image_filename(self.image, "width-200.format-webp")}
            width: 400, url: {get_test_image_filename(self.image, "width-400.format-webp")}
        """
        self.assertHTMLEqual(rendered, expected)

    def test_picture_assignment_render_as_is(self):
        rendered = self.render(
            '{% set bg=picture(myimage, "width-{200,400}|format-{jpeg,webp,gif}", sizes="100vw") %}{{ bg }}',
            {"myimage": self.image},
        )
        expected = self.render(
            '{{ picture(myimage, "width-{200,400}|format-{jpeg,webp,gif}", sizes="100vw") }}',
            {"myimage": self.image},
        )
        self.assertHTMLEqual(rendered, expected)

    def test_missing_picture(self):
        rendered = self.render(
            '{{ picture(myimage, "format-{jpeg,webp}") }}',
            {"myimage": self.bad_image},
        )
        expected = """
            <picture>
                <source srcset="/media/not-found" type="image/webp">
                <img
                    src="/media/not-found"
                    alt="missing image"
                    width="0"
                    height="0"
                >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)

    def test_invalid_character(self):
        with self.assertRaisesRegex(
            TemplateSyntaxError, "filter specs in 'picture' tag may only"
        ):
            self.render(
                '{{ picture(myimage, "fill-{20×20,40×40}", sizes="100vw") }}',
                {"myimage": self.image},
            )

    def test_custom_default_attrs(self):
        with unittest.mock.patch.object(
            apps.get_app_config("wagtailimages"),
            "default_attrs",
            new={"decoding": "async", "loading": "lazy"},
        ):
            rendered = self.render(
                '{{ picture(myimage, "format-{jpeg,webp}") }}',
                {"myimage": self.bad_image},
            )
            expected = """
                <picture>
                    <source srcset="/media/not-found" type="image/webp">
                    <img
                        src="/media/not-found"
                        alt="missing image"
                        width="0"
                        height="0"
                        decoding="async"
                        loading="lazy"
                    >
                </picture>
            """
            self.assertHTMLEqual(rendered, expected)

    def test_chaining_filterspecs(self):
        filename_jpeg = get_test_image_filename(
            self.image, "format-jpeg.jpegquality-40.webpquality-40"
        )
        filename_webp = get_test_image_filename(
            self.image, "format-webp.jpegquality-40.webpquality-40"
        )
        rendered = self.render(
            '{{ picture(myimage, "format-{jpeg,webp}|jpegquality-40|webpquality-40") }}',
            {"myimage": self.image},
        )
        expected = f"""
            <picture>
            <source srcset="{filename_webp}" type="image/webp">
            <img
                src="{filename_jpeg}"
                alt="Test image"
                width="640"
                height="480"
            >
            </picture>
        """
        self.assertHTMLEqual(rendered, expected)



from django.test import TestCase

from wagtail.test.utils import WagtailTestUtils
from wagtail.test.utils.template_tests import AdminTemplateTestUtils


class TestCustomListing(AdminTemplateTestUtils, WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def test_get(self):
        self.login()
        response = self.client.get("/admin/event_pages/")
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/pages/index.html")
        self.assertContains(response, "Event pages")
        self.assertContains(response, "Christmas")
        self.assertContains(response, "Saint Patrick")
        self.assertNotContains(response, "Welcome to the Wagtail test site!")
        self.assertBreadcrumbsItemsRendered(
            [{"url": "", "label": "Event pages"}],
            response.content,
        )
        soup = self.get_soup(response.content)
        breadcrumbs_icon = soup.select_one(".w-breadcrumbs__icon")
        self.assertIsNotNone(breadcrumbs_icon)
        use = breadcrumbs_icon.select_one("use")
        self.assertIsNotNone(use)
        self.assertEqual(use["href"], "#icon-calendar")

    def test_filter(self):
        self.login()
        response = self.client.get("/admin/event_pages/", {"audience": "private"})
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/pages/index.html")
        self.assertContains(response, "Event pages")
        self.assertNotContains(response, "Christmas")
        self.assertContains(response, "Saint Patrick")



from django.http import Http404, HttpResponse, HttpResponseForbidden
from django.utils.deprecation import MiddlewareMixin

from wagtail.models import Page
from wagtail.views import serve


class BlockDodgyUserAgentMiddleware(MiddlewareMixin):
    # Used to test that we're correctly handling responses returned from middleware during page
    # previews. If a client with user agent "EvilHacker" calls an admin view that performs a
    # preview, the request to /admin/... will pass this middleware, but the fake request used for
    # the preview (which keeps the user agent header, but uses the URL path of the front-end page)
    # will trigger a Forbidden response. In this case, the expected behaviour is to return that
    # response back to the user.

    def process_request(self, request):
        if (
            not request.path.startswith("/admin/")
            and request.headers.get("user-agent") == "EvilHacker"
        ):
            return HttpResponseForbidden("Forbidden")


class SimplePageViewInterceptorMiddleware(MiddlewareMixin):
    def process_view(self, request, view_func, view_args, view_kwargs):
        if serve == view_func:
            page = Page.find_for_request(request, *view_args, **view_kwargs)
            if page is None:
                raise Http404
            elif page.content == "Intercept me":
                return HttpResponse("Intercepted")



import json
import warnings

from django.core.serializers.json import DjangoJSONEncoder
from django.forms import Media, widgets
from django.utils.functional import cached_property

from wagtail.admin.rich_text.converters.contentstate import ContentstateConverter
from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.telepath import register
from wagtail.admin.telepath.widgets import WidgetAdapter
from wagtail.rich_text import features as feature_registry


class DraftailRichTextArea(widgets.HiddenInput):
    template_name = "wagtailadmin/widgets/draftail_rich_text_area.html"
    is_hidden = False

    # this class's constructor accepts a 'features' kwarg
    accepts_features = True

    # Draftail has its own commenting
    show_add_comment_button = False

    def __init__(self, *args, **kwargs):
        # note: this constructor will receive an 'options' kwarg taken from the WAGTAILADMIN_RICH_TEXT_EDITORS setting,
        # but we don't currently recognise any options from there (other than 'features', which is passed here as a separate kwarg)
        kwargs.pop("options", None)
        self.options = {}
        self.plugins = []

        self.features = kwargs.pop("features", None)
        if self.features is None:
            self.features = feature_registry.get_default_features()

        for feature in self.features:
            plugin = feature_registry.get_editor_plugin("draftail", feature)
            if plugin is None:
                warnings.warn(
                    f"Draftail received an unknown feature '{feature}'.",
                    category=RuntimeWarning,
                )
            else:
                plugin.construct_options(self.options)
                self.plugins.append(plugin)

        self.converter = ContentstateConverter(self.features)

        default_attrs = {
            "data-draftail-input": True,
            "data-controller": "w-init",
            "data-w-init-event-value": "w-draftail:init",
        }
        attrs = kwargs.get("attrs")
        if attrs:
            default_attrs.update(attrs)
        kwargs["attrs"] = default_attrs

        super().__init__(*args, **kwargs)

    def format_value(self, value):
        # Convert database rich text representation to the format required by
        # the input field
        value = super().format_value(value)

        if value is None:
            value = ""

        return self.converter.from_database_format(value)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context["widget"]["attrs"]["data-w-init-detail-value"] = json.dumps(
            self.options,
            cls=DjangoJSONEncoder,
        )
        return context

    def value_from_datadict(self, data, files, name):
        original_value = super().value_from_datadict(data, files, name)
        if original_value is None:
            return None
        return self.converter.to_database_format(original_value)

    @cached_property
    def media(self):
        media = Media(
            js=[
                versioned_static("wagtailadmin/js/draftail.js"),
            ],
            css={"all": [versioned_static("wagtailadmin/css/panels/draftail.css")]},
        )

        for plugin in self.plugins:
            media += plugin.media

        return media


class DraftailRichTextAreaAdapter(WidgetAdapter):
    js_constructor = "wagtail.widgets.DraftailRichTextArea"

    def js_args(self, widget):
        return [
            widget.options,
        ]


register(DraftailRichTextAreaAdapter(), DraftailRichTextArea)



from django.urls import path

from wagtail.contrib.forms.views import (
    DeleteSubmissionsView,
    FormPagesListView,
    get_submissions_list_view,
)

app_name = "wagtailforms"
urlpatterns = [
    path("", FormPagesListView.as_view(), name="index"),
    path(
        "results/", FormPagesListView.as_view(results_only=True), name="index_results"
    ),
    path(
        "submissions/<int:page_id>/", get_submissions_list_view, name="list_submissions"
    ),
    path(
        "submissions/<int:page_id>/results/",
        get_submissions_list_view,
        {"results_only": True},
        name="list_submissions_results",
    ),
    path(
        "submissions/<int:page_id>/delete/",
        DeleteSubmissionsView.as_view(),
        name="delete_submissions",
    ),
]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailFormsAppConfig(AppConfig):
    name = "wagtail.contrib.forms"
    label = "wagtailforms"
    verbose_name = _("Wagtail forms")
    default_auto_field = "django.db.models.AutoField"



# Generated by Django 4.2.13 on 2024-08-05 22:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailimages", "0026_delete_uploadedimage"),
    ]

    operations = [
        migrations.AddField(
            model_name="image",
            name="description",
            field=models.CharField(
                blank=True, default="", max_length=255, verbose_name="description"
            ),
        ),
    ]



from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse
from django.utils.functional import cached_property

from wagtail.admin.menu import WagtailMenuRegisterable, WagtailMenuRegisterableGroup


class ViewSet(WagtailMenuRegisterable):
    """
    Defines a viewset to be registered with the Wagtail admin.

    All properties of the viewset can be defined as class-level attributes, or passed as
    keyword arguments to the constructor (in which case they will override any class-level
    attributes). Additionally, the :attr:`name` property can be passed as the first positional
    argument to the constructor.

    For more information on how to use this class, see :ref:`using_base_viewset`.
    """

    #: A special value that, when passed in a kwargs dict to construct a view, indicates that
    #: the attribute should not be written and should instead be left as the view's initial value
    UNDEFINED = object()

    #: A name for this viewset, used as the default URL prefix and namespace.
    name = None

    #: The icon to use across the views.
    icon = ""

    def __init__(self, name=None, **kwargs):
        if name:
            self.__dict__["name"] = name

        for key, value in kwargs.items():
            self.__dict__[key] = value

    def get_common_view_kwargs(self, **kwargs):
        """
        Returns a dictionary of keyword arguments to be passed to all views within this viewset.
        """
        return kwargs

    def construct_view(self, view_class, **kwargs):
        """
        Wrapper for view_class.as_view() which passes the kwargs returned from get_common_view_kwargs
        in addition to any kwargs passed to this method. Items from get_common_view_kwargs will be
        filtered to only include those that are valid for the given view_class.
        """
        merged_kwargs = self.get_common_view_kwargs()
        merged_kwargs.update(kwargs)
        filtered_kwargs = {
            key: value
            for key, value in merged_kwargs.items()
            if hasattr(view_class, key) and value is not self.UNDEFINED
        }
        return view_class.as_view(**filtered_kwargs)

    def inject_view_methods(self, view_class, method_names):
        """
        Check for the presence of any of the named methods on this viewset. If any are found,
        create a subclass of view_class that overrides those methods to call the implementation
        on this viewset instead. Otherwise, return view_class unmodified.
        """
        viewset = self
        overrides = {}

        def make_view_method(viewset_method):
            def _view_method(self, *args, **kwargs):
                return viewset_method(*args, **kwargs)

            return _view_method

        for method_name in method_names:
            viewset_method = getattr(viewset, method_name, None)
            if viewset_method:
                view_method = make_view_method(viewset_method)
                view_method.__name__ = method_name
                overrides[method_name] = view_method

        if overrides:
            return type(view_class.__name__, (view_class,), overrides)
        else:
            return view_class

    @cached_property
    def url_prefix(self):
        """
        The preferred URL prefix for views within this viewset. When registered through
        Wagtail's :ref:`register_admin_viewset` hook, this will be used as the URL path component
        following ``/admin/``. Other URL registration mechanisms (e.g. editing ``urls.py`` manually)
        may disregard this and use a prefix of their own choosing.

        Defaults to the viewset's ``name``.
        """
        if not self.name:
            raise ImproperlyConfigured(
                "ViewSet %r must provide a `name` property" % self
            )
        return self.name

    @cached_property
    def url_namespace(self):
        """
        The URL namespace for views within this viewset. Will be used internally as the
        application namespace for the viewset's URLs, and generally be the instance namespace
        too.

        Defaults to the viewset's ``name``.
        """
        if not self.name:
            raise ImproperlyConfigured(
                "ViewSet %r must provide a `name` property" % self
            )
        return self.name

    def on_register(self):
        """
        Called when the viewset is registered; subclasses can override this to perform additional setup.
        """
        self.register_menu_item()

    def get_urlpatterns(self):
        """
        Returns a set of URL routes to be registered with the Wagtail admin.
        """
        return []

    def get_url_name(self, view_name):
        """
        Returns the namespaced URL name for the given view.
        """
        return self.url_namespace + ":" + view_name

    @cached_property
    def menu_icon(self):
        return self.icon

    @cached_property
    def menu_url(self):
        return reverse(self.get_url_name(self.get_urlpatterns()[0].name))


class ViewSetGroup(WagtailMenuRegisterableGroup):
    """
    A container for grouping together multiple :class:`ViewSet` instances.
    Creates a menu item with a submenu for accessing the main URL for each instances.

    For more information on how to use this class, see :ref:`using_base_viewsetgroup`.
    """

    def on_register(self):
        self.register_menu_item()



# Generated by Django 4.2.1 on 2023-06-21 11:44

from django import VERSION as DJANGO_VERSION
from django.db import migrations, models


check_constraint_condition = models.Q(
    ("permission__isnull", False),
    ("permission_type__isnull", False),
    _connector="OR",
)

check_constraint_kwargs = {"name": "permission_or_permission_type_not_null"}

if DJANGO_VERSION >= (5, 1):
    check_constraint_kwargs["condition"] = check_constraint_condition
else:
    check_constraint_kwargs["check"] = check_constraint_condition


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0086_populate_grouppagepermission_permission"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="grouppagepermission",
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name="grouppagepermission",
            constraint=models.CheckConstraint(**check_constraint_kwargs),
        ),
        migrations.AddConstraint(
            model_name="grouppagepermission",
            constraint=models.UniqueConstraint(
                fields=("group", "page", "permission"),
                name="unique_permission",
            ),
        ),
        migrations.AddConstraint(
            model_name="grouppagepermission",
            constraint=models.UniqueConstraint(
                fields=("group", "page", "permission_type"),
                name="unique_permission_type",
            ),
        ),
    ]



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class WagtailStyleGuideAppConfig(AppConfig):
    name = "wagtail.contrib.styleguide"
    label = "wagtailstyleguide"
    verbose_name = _("Wagtail style guide")



from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.test import TestCase
from django.urls import reverse
from django.utils.http import urlencode

from wagtail.models import GroupPagePermission, Page
from wagtail.test.testapp.models import EventPage
from wagtail.test.utils import WagtailTestUtils


class TestContentTypeUse(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.user = self.login()
        self.christmas_page = EventPage.objects.get(title="Christmas")

    def test_with_no_permission(self):
        request_url = reverse(
            "wagtailadmin_pages:type_use", args=("tests", "eventpage")
        )
        group = Group.objects.create(name="test group")
        self.user.is_superuser = False
        self.user.save()
        self.user.groups.add(group)
        group.permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            )
        )
        # No GroupPagePermission created

        response = self.client.get(request_url)
        self.assertEqual(response.status_code, 302)
        self.assertRedirects(response, reverse("wagtailadmin_home"))

    def test_with_minimal_permissions(self):
        request_url = reverse(
            "wagtailadmin_pages:type_use", args=("tests", "eventpage")
        )
        group = Group.objects.create(name="test group")
        self.user.is_superuser = False
        self.user.save()
        self.user.groups.add(group)
        group.permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            ),
        )
        GroupPagePermission.objects.create(
            group=group,
            page=Page.objects.first(),
            permission_type="change",
        )

        response = self.client.get(request_url)
        self.assertEqual(response.status_code, 200)

    def test_content_type_use(self):
        # Get use of event page
        request_url = reverse(
            "wagtailadmin_pages:type_use", args=("tests", "eventpage")
        )
        response = self.client.get(request_url)

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing.html")
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing_results.html")
        self.assertContains(response, "Christmas")

        # Links to 'delete' etc should include a 'next' URL parameter pointing back here
        delete_url = (
            reverse("wagtailadmin_pages:delete", args=(self.christmas_page.id,))
            + "?"
            + urlencode({"next": request_url})
        )
        self.assertContains(response, delete_url)
        self.assertContains(response, "data-bulk-action-select-all-checkbox")

        with self.assertNumQueries(33):
            self.client.get(request_url)

    def test_content_type_use_results(self):
        # Get the results view of event page use, with search and filter
        ameristralia_page = EventPage.objects.get(title="Ameristralia Day")
        user = get_user_model().objects.get(email="eventmoderator@example.com")
        request_url = reverse(
            "wagtailadmin_pages:type_use_results",
            args=("tests", "eventpage"),
        )
        response = self.client.get(
            request_url,
            data={"q": "Ameristralia", "owner": user.pk},
        )

        # Check response
        self.assertEqual(response.status_code, 200)
        self.assertTemplateNotUsed(response, "wagtailadmin/generic/listing.html")
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing_results.html")
        self.assertContains(response, "Ameristralia Day")

        # Links to 'delete' etc should include a 'next' URL parameter pointing back
        # to the full view, not the results-only view
        full_view_url = reverse(
            "wagtailadmin_pages:type_use",
            args=("tests", "eventpage"),
        )
        delete_url = (
            reverse("wagtailadmin_pages:delete", args=(ameristralia_page.id,))
            + "?"
            + urlencode({"next": full_view_url})
        )
        self.assertContains(response, delete_url)
        self.assertContains(response, "data-bulk-action-select-all-checkbox")



from django.contrib.contenttypes.models import ContentType

from wagtail.coreutils import resolve_model_string

content_types_to_register = []
CONTENT_TYPE_ORDER = {}


def register(model, **kwargs):
    """
    Registers order against the model content_type, used to
    control the order the models and its permissions appear
    in the groups object permission editor
    """
    order = kwargs.pop("order", None)
    if order is not None:
        # We typically call this at application startup, when the database may not be ready,
        # and so we can't look up the content type yet. Instead we will queue up the
        # (model, order) pair to be processed when the lookup is requested.
        content_types_to_register.append((model, order))


def get_content_type_order_lookup():
    if content_types_to_register:
        for model, order in content_types_to_register:
            content_type = ContentType.objects.get_for_model(
                resolve_model_string(model)
            )
            CONTENT_TYPE_ORDER[content_type.id] = order
        content_types_to_register.clear()
    return CONTENT_TYPE_ORDER



from django.core.management.base import BaseCommand

from wagtail.models import Page


class Command(BaseCommand):
    def add_arguments(self, parser):
        # Positional arguments
        parser.add_argument("from_id", type=int)
        parser.add_argument("to_id", type=int)

    def handle(self, *args, **options):
        # Get pages
        from_page = Page.objects.get(pk=options["from_id"])
        to_page = Page.objects.get(pk=options["to_id"])
        pages = from_page.get_children()

        # Move the pages
        self.stdout.write(
            "Moving "
            + str(len(pages))
            + ' pages from "'
            + from_page.title
            + '" to "'
            + to_page.title
            + '"'
        )
        for page in pages:
            page.move(to_page, pos="last-child")

        self.stdout.write("Done")



# Generated by Django 3.1.8 on 2021-06-22 14:19

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailsearchpromotions", "0002_capitalizeverbose"),
    ]

    operations = [
        migrations.CreateModel(
            name="Query",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("query_string", models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="QueryDailyHits",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField()),
                ("hits", models.IntegerField(default=0)),
                (
                    "query",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="daily_hits",
                        to="wagtailsearchpromotions.query",
                    ),
                ),
            ],
            options={
                "verbose_name": "Query Daily Hits",
                "verbose_name_plural": "Query Daily Hits",
                "unique_together": {("query", "date")},
            },
        ),
    ]



# Generated by Django 3.1.3 on 2020-11-27 01:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0059_apply_collection_ordering"),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name="workflowstate",
            name="unique_in_progress_workflow",
        ),
        migrations.AddConstraint(
            model_name="workflowstate",
            constraint=models.UniqueConstraint(
                condition=models.Q(status__in=("in_progress", "needs_changes")),
                fields=("page",),
                name="unique_in_progress_workflow",
            ),
        ),
    ]



import django_filters
from django.core.exceptions import PermissionDenied
from django.db.models import Q
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy

from wagtail.admin.views.generic import history
from wagtail.admin.views.pages.utils import (
    GenericPageBreadcrumbsMixin,
)
from wagtail.admin.widgets import BooleanRadioSelect
from wagtail.models import Page, PageLogEntry
from wagtail.permissions import page_permission_policy


class PageHistoryFilterSet(history.HistoryFilterSet):
    is_commenting_action = django_filters.BooleanFilter(
        label=gettext_lazy("Is commenting action"),
        method="filter_is_commenting_action",
        widget=BooleanRadioSelect,
    )

    def filter_is_commenting_action(self, queryset, name, value):
        if value is None:
            return queryset

        q = Q(action__startswith="wagtail.comments")
        if value is False:
            q = ~q

        return queryset.filter(q)


class PageWorkflowHistoryViewMixin:
    model = Page
    pk_url_kwarg = "page_id"
    edit_url_name = "wagtailadmin_pages:edit"

    def dispatch(self, request, *args, **kwargs):
        if not self.object.permissions_for_user(request.user).can_edit():
            raise PermissionDenied
        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        return super().get_context_data(**kwargs, page=self.object)


class WorkflowHistoryView(
    PageWorkflowHistoryViewMixin,
    GenericPageBreadcrumbsMixin,
    history.WorkflowHistoryView,
):
    header_icon = "doc-empty-inverse"
    workflow_history_detail_url_name = "wagtailadmin_pages:workflow_history_detail"


class WorkflowHistoryDetailView(
    PageWorkflowHistoryViewMixin,
    GenericPageBreadcrumbsMixin,
    history.WorkflowHistoryDetailView,
):
    header_icon = "doc-empty-inverse"
    workflow_history_url_name = "wagtailadmin_pages:workflow_history"
    breadcrumbs_items_to_take = 2


class PageHistoryView(GenericPageBreadcrumbsMixin, history.HistoryView):
    template_name = "wagtailadmin/pages/history.html"
    filterset_class = PageHistoryFilterSet
    model = Page
    pk_url_kwarg = "page_id"
    permission_policy = page_permission_policy
    any_permission_required = {
        "add",
        "change",
        "publish",
        "bulk_delete",
        "lock",
        "unlock",
    }
    history_url_name = "wagtailadmin_pages:history"
    history_results_url_name = "wagtailadmin_pages:history_results"
    edit_url_name = "wagtailadmin_pages:edit"
    revisions_view_url_name = "wagtailadmin_pages:revisions_view"
    revisions_revert_url_name = "wagtailadmin_pages:revisions_revert"
    revisions_compare_url_name = "wagtailadmin_pages:revisions_compare"
    revisions_unschedule_url_name = "wagtailadmin_pages:revisions_unschedule"

    def get_object(self):
        return get_object_or_404(Page, id=self.pk).specific

    def get_page_subtitle(self):
        return self.object.get_admin_display_title()

    def user_can_unschedule(self):
        return self.object.permissions_for_user(self.request.user).can_unschedule()

    def get_base_queryset(self):
        return self._annotate_queryset(PageLogEntry.objects.filter(page=self.object))

    def _annotate_queryset(self, queryset):
        return super()._annotate_queryset(queryset).select_related("page")



# Generated by Django 4.0.3 on 2022-07-04 13:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0076_modellogentry_revision"),
        ("tests", "0001_squashed_0073_revisablechildmodel_secret_text"),
    ]

    operations = [
        migrations.CreateModel(
            name="FileGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("file", models.FileField(upload_to="")),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="FileSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("file", models.FileField(upload_to="")),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="IconGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="IconSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ImportantPagesGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "general_terms_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "privacy_policy_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "sign_up_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ImportantPagesSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "general_terms_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "privacy_policy_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "sign_up_page",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="NotYetRegisteredGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="NotYetRegisteredSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=100)),
                ("email", models.EmailField(max_length=50)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="TestSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=100)),
                ("email", models.EmailField(max_length=50)),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="PanelGenericSettings",
            fields=[
                (
                    "testgenericsetting_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.testgenericsetting",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("tests.testgenericsetting",),
        ),
        migrations.CreateModel(
            name="PanelSiteSettings",
            fields=[
                (
                    "testsitesetting_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.testsitesetting",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("tests.testsitesetting",),
        ),
        migrations.CreateModel(
            name="TabbedGenericSettings",
            fields=[
                (
                    "testgenericsetting_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.testgenericsetting",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("tests.testgenericsetting",),
        ),
        migrations.CreateModel(
            name="TabbedSiteSettings",
            fields=[
                (
                    "testsitesetting_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="tests.testsitesetting",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("tests.testsitesetting",),
        ),
        migrations.DeleteModel(
            name="FileUploadSetting",
        ),
        migrations.DeleteModel(
            name="IconSetting",
        ),
        migrations.DeleteModel(
            name="ImportantPages",
        ),
        migrations.DeleteModel(
            name="NotYetRegisteredSetting",
        ),
        migrations.DeleteModel(
            name="PanelSettings",
        ),
        migrations.DeleteModel(
            name="TabbedSettings",
        ),
        migrations.DeleteModel(
            name="TestSetting",
        ),
    ]



import json

from django.urls import reverse

from wagtail.api.v2.tests.test_images import (
    TestImageDetail,
    TestImageListing,
    TestImageListingSearch,
)
from wagtail.images import get_image_model
from wagtail.images.tests.utils import get_test_image_file

from .utils import AdminAPITestCase


class TestAdminImageListing(AdminAPITestCase, TestImageListing):
    fixtures = ["demosite.json"]

    def get_response(self, **params):
        return self.client.get(reverse("wagtailadmin_api:images:listing"), params)

    def get_image_id_list(self, content):
        return [image["id"] for image in content["items"]]

    # BASIC TESTS

    def test_basic(self):
        response = self.get_response()

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["Content-type"], "application/json")

        # Will crash if the JSON is invalid
        content = json.loads(response.content.decode("UTF-8"))

        # Check that the meta section is there
        self.assertIn("meta", content)
        self.assertIsInstance(content["meta"], dict)

        # Check that the total count is there and correct
        self.assertIn("total_count", content["meta"])
        self.assertIsInstance(content["meta"]["total_count"], int)
        self.assertEqual(
            content["meta"]["total_count"], get_image_model().objects.count()
        )

        # Check that the items section is there
        self.assertIn("items", content)
        self.assertIsInstance(content["items"], list)

        # Check that each image has a meta section with type, detail_url and tags attributes
        for image in content["items"]:
            self.assertIn("meta", image)
            self.assertIsInstance(image["meta"], dict)
            self.assertEqual(
                set(image["meta"].keys()),
                {"type", "detail_url", "tags", "download_url"},
            )

            # Type should always be wagtailimages.Image
            self.assertEqual(image["meta"]["type"], "wagtailimages.Image")

            # Check detail url
            self.assertEqual(
                image["meta"]["detail_url"],
                "http://localhost/admin/api/main/images/%d/" % image["id"],
            )

    #  FIELDS

    def test_fields_default(self):
        response = self.get_response()
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "meta", "title", "width", "height", "thumbnail"},
            )
            self.assertEqual(
                set(image["meta"].keys()),
                {"type", "detail_url", "download_url", "tags"},
            )

    def test_fields(self):
        response = self.get_response(fields="width,height")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "meta", "title", "width", "height", "thumbnail"},
            )
            self.assertEqual(
                set(image["meta"].keys()),
                {"type", "detail_url", "download_url", "tags"},
            )

    def test_remove_fields(self):
        response = self.get_response(fields="-title")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()), {"id", "meta", "width", "height", "thumbnail"}
            )

    def test_remove_meta_fields(self):
        response = self.get_response(fields="-tags")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "meta", "title", "width", "height", "thumbnail"},
            )
            self.assertEqual(
                set(image["meta"].keys()), {"type", "detail_url", "download_url"}
            )

    def test_remove_all_meta_fields(self):
        response = self.get_response(fields="-type,-detail_url,-tags")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "title", "width", "height", "thumbnail", "meta"},
            )

    def test_remove_id_field(self):
        response = self.get_response(fields="-id")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()), {"meta", "title", "width", "height", "thumbnail"}
            )

    def test_all_fields(self):
        response = self.get_response(fields="*")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "meta", "title", "width", "height", "thumbnail"},
            )
            self.assertEqual(
                set(image["meta"].keys()),
                {"type", "detail_url", "tags", "download_url"},
            )

    def test_all_fields_then_remove_something(self):
        response = self.get_response(fields="*,-title,-tags")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()), {"id", "meta", "width", "height", "thumbnail"}
            )
            self.assertEqual(
                set(image["meta"].keys()), {"type", "detail_url", "download_url"}
            )

    def test_fields_tags(self):
        response = self.get_response(fields="tags")
        content = json.loads(response.content.decode("UTF-8"))

        for image in content["items"]:
            self.assertEqual(
                set(image.keys()),
                {"id", "meta", "title", "width", "height", "thumbnail"},
            )
            self.assertEqual(
                set(image["meta"].keys()),
                {"type", "detail_url", "tags", "download_url"},
            )
            self.assertIsInstance(image["meta"]["tags"], list)


class TestAdminImageListingSearch(AdminAPITestCase, TestImageListingSearch):
    fixtures = ["demosite.json"]

    def get_response(self, **params):
        return self.client.get(reverse("wagtailadmin_api:images:listing"), params)

    def get_image_id_list(self, content):
        return [image["id"] for image in content["items"]]


class TestAdminImageDetail(AdminAPITestCase, TestImageDetail):
    fixtures = ["demosite.json"]

    def get_response(self, image_id, **params):
        return self.client.get(
            reverse("wagtailadmin_api:images:detail", args=(image_id,)), params
        )

    def test_basic(self):
        response = self.get_response(5)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["Content-type"], "application/json")

        # Will crash if the JSON is invalid
        content = json.loads(response.content.decode("UTF-8"))

        # Check the id field
        self.assertIn("id", content)
        self.assertEqual(content["id"], 5)

        # Check that the meta section is there
        self.assertIn("meta", content)
        self.assertIsInstance(content["meta"], dict)

        # Check the meta type
        self.assertIn("type", content["meta"])
        self.assertEqual(content["meta"]["type"], "wagtailimages.Image")

        # Check the meta detail_url
        self.assertIn("detail_url", content["meta"])
        self.assertEqual(
            content["meta"]["detail_url"], "http://localhost/admin/api/main/images/5/"
        )

        # Check the thumbnail

        # Note: This is None because the source image doesn't exist
        #       See test_thumbnail below for working example
        self.assertIn("thumbnail", content)
        self.assertEqual(content["thumbnail"], {"error": "SourceImageIOError"})

        # Check the title field
        self.assertIn("title", content)
        self.assertEqual(content["title"], "James Joyce")

        # Check the width and height fields
        self.assertIn("width", content)
        self.assertIn("height", content)
        self.assertEqual(content["width"], 500)
        self.assertEqual(content["height"], 392)

        # Check the tags field
        self.assertIn("tags", content["meta"])
        self.assertEqual(content["meta"]["tags"], [])

    def test_thumbnail(self):
        # Add a new image with source file
        image = get_image_model().objects.create(
            title="Test image",
            file=get_test_image_file(),
        )

        response = self.get_response(image.id)
        content = json.loads(response.content.decode("UTF-8"))

        self.assertIn("thumbnail", content)
        self.assertEqual(content["thumbnail"]["width"], 165)
        self.assertEqual(content["thumbnail"]["height"], 123)
        self.assertTrue(content["thumbnail"]["url"].startswith("/media/images/test"))

        # Check that source_image_error didn't appear
        self.assertNotIn("source_image_error", content["meta"])


# Overwrite imported test cases do Django doesn't run them
TestImageDetail = None
TestImageListing = None



# Generated by Django 5.0.9 on 2024-10-16 05:17

import django.db.models.deletion
import wagtail.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0043_customimage_description"),
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomPreviewSizesModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
            ],
            bases=(wagtail.models.PreviewableMixin, models.Model),
        ),
        migrations.CreateModel(
            name="CustomPreviewSizesPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page",),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailembeds", "0002_add_verbose_names"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="embed",
            options={"verbose_name": "embed"},
        ),
    ]



from django.http import Http404
from rest_framework import serializers

from wagtail.api.v2.filters import FieldsFilter, OrderingFilter, SearchFilter
from wagtail.api.v2.serializers import BaseSerializer
from wagtail.api.v2.views import BaseAPIViewSet
from wagtail.contrib.redirects.middleware import get_redirect
from wagtail.contrib.redirects.models import Redirect


class RedirectSerializer(BaseSerializer):
    location = serializers.CharField(source="link")


class RedirectsAPIViewSet(BaseAPIViewSet):
    base_serializer_class = RedirectSerializer
    filter_backends = [FieldsFilter, OrderingFilter, SearchFilter]
    body_fields = BaseAPIViewSet.body_fields + ["old_path", "location"]
    name = "redirects"
    model = Redirect

    listing_default_fields = BaseAPIViewSet.listing_default_fields + [
        "old_path",
        "location",
    ]

    def find_object(self, queryset, request):
        if "html_path" in request.GET:
            redirect = get_redirect(
                request,
                request.GET["html_path"],
            )

            if redirect is None:
                raise Http404
            else:
                return redirect

        return super().find_object(queryset, request)



from collections import OrderedDict

from django.conf import settings
from django.http import Http404
from django.urls import path
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response

from wagtail.api.v2.views import PagesAPIViewSet
from wagtail.models import Page

from .actions.convert_alias import ConvertAliasPageAPIAction
from .actions.copy import CopyPageAPIAction
from .actions.copy_for_translation import CopyForTranslationAPIAction
from .actions.create_alias import CreatePageAliasAPIAction
from .actions.delete import DeletePageAPIAction
from .actions.move import MovePageAPIAction
from .actions.publish import PublishPageAPIAction
from .actions.revert_to_page_revision import RevertToPageRevisionAPIAction
from .actions.unpublish import UnpublishPageAPIAction
from .filters import ForExplorerFilter, HasChildrenFilter
from .serializers import AdminPageSerializer


class PagesAdminAPIViewSet(PagesAPIViewSet):
    base_serializer_class = AdminPageSerializer
    authentication_classes = [SessionAuthentication]

    actions = {
        "convert_alias": ConvertAliasPageAPIAction,
        "copy": CopyPageAPIAction,
        "delete": DeletePageAPIAction,
        "publish": PublishPageAPIAction,
        "unpublish": UnpublishPageAPIAction,
        "move": MovePageAPIAction,
        "copy_for_translation": CopyForTranslationAPIAction,
        "create_alias": CreatePageAliasAPIAction,
        "revert_to_page_revision": RevertToPageRevisionAPIAction,
    }

    # Add has_children and for_explorer filters
    filter_backends = PagesAPIViewSet.filter_backends + [
        HasChildrenFilter,
        ForExplorerFilter,
    ]

    meta_fields = PagesAPIViewSet.meta_fields + [
        "latest_revision_created_at",
        "status",
        "children",
        "descendants",
        "parent",
        "ancestors",
        "translations",
    ]

    body_fields = PagesAPIViewSet.body_fields + [
        "admin_display_title",
    ]

    listing_default_fields = PagesAPIViewSet.listing_default_fields + [
        "latest_revision_created_at",
        "status",
        "children",
        "admin_display_title",
    ]

    # Allow the parent field to appear on listings
    detail_only_fields = []

    known_query_parameters = PagesAPIViewSet.known_query_parameters.union(
        ["for_explorer", "has_children"]
    )

    @classmethod
    def get_detail_default_fields(cls, model):
        detail_default_fields = super().get_detail_default_fields(model)

        # When i18n is disabled, remove "translations" from default fields
        if not getattr(settings, "WAGTAIL_I18N_ENABLED", False):
            detail_default_fields.remove("translations")

        return detail_default_fields

    def get_root_page(self):
        """
        Returns the page that is used when the `&child_of=root` filter is used.
        """
        return Page.get_first_root_node()

    def get_base_queryset(self):
        """
        Returns a queryset containing all pages that can be seen by this user.

        This is used as the base for get_queryset and is also used to find the
        parent pages when using the child_of and descendant_of filters as well.
        """
        return Page.objects.all()

    def get_queryset(self):
        queryset = super().get_queryset()

        # Hide root page
        # TODO: Add "include_root" flag
        queryset = queryset.exclude(depth=1).defer_streamfields().specific()

        return queryset

    def get_type_info(self):
        types = OrderedDict()

        for name, model in self.seen_types.items():
            types[name] = OrderedDict(
                [
                    ("verbose_name", model._meta.verbose_name),
                    ("verbose_name_plural", model._meta.verbose_name_plural),
                ]
            )

        return types

    def listing_view(self, request):
        response = super().listing_view(request)
        response.data["__types"] = self.get_type_info()
        return response

    def detail_view(self, request, pk):
        response = super().detail_view(request, pk)
        response.data["__types"] = self.get_type_info()
        return response

    def action_view(self, request, pk, action_name):
        instance = self.get_object()

        if action_name not in self.actions:
            raise Http404(f"unrecognised action '{action_name}'")

        action = self.actions[action_name](self, request)
        action_data = action.serializer(data=request.data)

        if not action_data.is_valid():
            return Response(action_data.errors, status=400)

        return action.execute(instance, action_data.data)

    @classmethod
    def get_urlpatterns(cls):
        """
        This returns a list of URL patterns for the endpoint
        """
        urlpatterns = super().get_urlpatterns()
        urlpatterns.extend(
            [
                path(
                    "<int:pk>/action/<str:action_name>/",
                    cls.as_view({"post": "action_view"}),
                    name="action",
                ),
            ]
        )
        return urlpatterns



from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from treebeard.mp_tree import MP_Node

from wagtail.query import TreeQuerySet
from wagtail.search import index

from .view_restrictions import BaseViewRestriction


class CollectionQuerySet(TreeQuerySet):
    def get_min_depth(self):
        return self.aggregate(models.Min("depth"))["depth__min"] or 2

    def get_indented_choices(self):
        """
        Return a list of (id, label) tuples for use as a list of choices in a collection chooser
        dropdown, where the label is formatted with get_indented_name to provide a tree layout.
        The indent level is chosen to place the minimum-depth collection at indent 0.
        """
        min_depth = self.get_min_depth()
        return [
            (collection.pk, collection.get_indented_name(min_depth, html=True))
            for collection in self
        ]


class BaseCollectionManager(models.Manager):
    def get_queryset(self):
        return CollectionQuerySet(self.model).order_by("path")


CollectionManager = BaseCollectionManager.from_queryset(CollectionQuerySet)


class CollectionViewRestriction(BaseViewRestriction):
    collection = models.ForeignKey(
        "Collection",
        verbose_name=_("collection"),
        related_name="view_restrictions",
        on_delete=models.CASCADE,
    )

    passed_view_restrictions_session_key = "passed_collection_view_restrictions"

    class Meta:
        verbose_name = _("collection view restriction")
        verbose_name_plural = _("collection view restrictions")


class Collection(MP_Node):
    """
    A location in which resources such as images and documents can be grouped
    """

    name = models.CharField(max_length=255, verbose_name=_("name"))

    objects = CollectionManager()
    # Tell treebeard to order Collections' paths such that they are ordered by name at each level.
    node_order_by = ["name"]

    def __str__(self):
        return self.name

    def get_ancestors(self, inclusive=False):
        return Collection.objects.ancestor_of(self, inclusive)

    def get_descendants(self, inclusive=False):
        return Collection.objects.descendant_of(self, inclusive)

    def get_siblings(self, inclusive=True):
        return Collection.objects.sibling_of(self, inclusive)

    def get_next_siblings(self, inclusive=False):
        return self.get_siblings(inclusive).filter(path__gte=self.path).order_by("path")

    def get_prev_siblings(self, inclusive=False):
        return (
            self.get_siblings(inclusive).filter(path__lte=self.path).order_by("-path")
        )

    def get_view_restrictions(self):
        """Return a query set of all collection view restrictions that apply to this collection"""
        return CollectionViewRestriction.objects.filter(
            collection__in=self.get_ancestors(inclusive=True)
        )

    def get_indented_name(self, indentation_start_depth=2, html=False):
        """
        Renders this Collection's name as a formatted string that displays its hierarchical depth via indentation.
        If indentation_start_depth is supplied, the Collection's depth is rendered relative to that depth.
        indentation_start_depth defaults to 2, the depth of the first non-Root Collection.
        Pass html=True to get an HTML representation, instead of the default plain-text.

        Example text output: "    ↳ Pies"
        Example HTML output: "&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Pies"
        """
        display_depth = self.depth - indentation_start_depth
        # A Collection with a display depth of 0 or less (Root's can be -1), should have no indent.
        if display_depth <= 0:
            return self.name

        # Indent each level of depth by 4 spaces (the width of the ↳ character in our admin font), then add ↳
        # before adding the name.
        if html:
            # NOTE: &#x21b3 is the hex HTML entity for ↳.
            return format_html(
                "{indent}{icon} {name}",
                indent=mark_safe("&nbsp;" * 4 * display_depth),
                icon=mark_safe("&#x21b3"),
                name=self.name,
            )
        # Output unicode plain-text version
        return "{}↳ {}".format(" " * 4 * display_depth, self.name)

    class Meta:
        verbose_name = _("collection")
        verbose_name_plural = _("collections")


def get_root_collection_id():
    return Collection.get_first_root_node().id


class CollectionMember(models.Model):
    """
    Base class for models that are categorised into collections
    """

    collection = models.ForeignKey(
        Collection,
        default=get_root_collection_id,
        verbose_name=_("collection"),
        related_name="+",
        on_delete=models.CASCADE,
    )

    search_fields = [
        index.FilterField("collection"),
    ]

    class Meta:
        abstract = True


class GroupCollectionPermissionManager(models.Manager):
    def get_by_natural_key(self, group, collection, permission):
        return self.get(group=group, collection=collection, permission=permission)


class GroupCollectionPermission(models.Model):
    """
    A rule indicating that a group has permission for some action (e.g. "create document")
    within a specified collection.
    """

    group = models.ForeignKey(
        Group,
        verbose_name=_("group"),
        related_name="collection_permissions",
        on_delete=models.CASCADE,
    )
    collection = models.ForeignKey(
        Collection,
        verbose_name=_("collection"),
        related_name="group_permissions",
        on_delete=models.CASCADE,
    )
    permission = models.ForeignKey(
        Permission, verbose_name=_("permission"), on_delete=models.CASCADE
    )

    def __str__(self):
        return "Group %d ('%s') has permission '%s' on collection %d ('%s')" % (
            self.group.id,
            self.group,
            self.permission,
            self.collection.id,
            self.collection,
        )

    def natural_key(self):
        return (self.group, self.collection, self.permission)

    objects = GroupCollectionPermissionManager()

    class Meta:
        unique_together = ("group", "collection", "permission")
        verbose_name = _("group collection permission")
        verbose_name_plural = _("group collection permissions")


class UploadedFile(models.Model):
    """
    Temporary storage for media fields uploaded through the multiple image/document uploader.
    When validation rules (e.g. required metadata fields) prevent creating an Image/Document object from the file alone.
    In this case, the file is stored against this model, to be turned into an Image/Document object once the full form
    has been filled in.
    """

    for_content_type = models.ForeignKey(
        ContentType,
        verbose_name=_("for content type"),
        related_name="uploads",
        on_delete=models.CASCADE,
        null=True,
    )
    file = models.FileField(upload_to="wagtail_uploads", max_length=200)
    uploaded_by_user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_("uploaded by user"),
        null=True,
        blank=True,
        editable=False,
        on_delete=models.SET_NULL,
    )



from urllib.parse import parse_qs, urlparse, urlunparse

from django.conf import settings
from django.utils.http import urlencode
from django.utils.translation import gettext_lazy as _

from wagtail.compat import AUTH_USER_APP_LABEL, AUTH_USER_MODEL_NAME
from wagtail.coreutils import safe_md5

delete_user_perm = "{}.delete_{}".format(
    AUTH_USER_APP_LABEL, AUTH_USER_MODEL_NAME.lower()
)


def user_can_delete_user(current_user, user_to_delete):
    if not current_user.has_perm(delete_user_perm):
        return False

    if current_user == user_to_delete:
        # users may not delete themselves
        return False

    if user_to_delete.is_superuser and not current_user.is_superuser:
        # ordinary users may not delete superusers
        return False

    return True


def get_gravatar_url(email, size=50, default_params={"d": "mp"}):
    """
    See https://gravatar.com/site/implement/images/ for Gravatar image options.

    Example usage:

    .. code-block:: python

        # Basic usage
        gravatar_url = get_gravatar_url('user@example.com')

        # Customize size and default image
        gravatar_url = get_gravatar_url(
            'user@example.com',
            size=100,
            default_params={'d': 'robohash', 'f': 'y'}
        )

    Note:
        If any parameter in ``default_params`` also exists in the provider URL,
        it will be overridden by the provider URL's query parameter.
    """

    gravatar_provider_url = getattr(
        settings, "WAGTAIL_GRAVATAR_PROVIDER_URL", "//www.gravatar.com/avatar"
    )

    if (not email) or (gravatar_provider_url is None):
        return None

    parsed_url = urlparse(gravatar_provider_url)

    params = {
        **default_params,
        **(parse_qs(parsed_url.query or "")),
        # requested at retina size by default and scaled down at point of use with css
        "s": int(size) * 2,
    }

    email_hash = safe_md5(
        email.lower().encode("utf-8"), usedforsecurity=False
    ).hexdigest()

    parsed_url = parsed_url._replace(
        path=f"{parsed_url.path.rstrip('/')}/{email_hash}",
        query=urlencode(params, doseq=True),
    )

    gravatar_url = urlunparse(parsed_url)

    return gravatar_url


def get_deleted_user_display_name(user_id):
    # Use a string placeholder as the user id could be non-numeric
    return _("user %(id)s (deleted)") % {"id": user_id}



# Generated by Django 4.2.6 on 2023-10-26 15:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0090_remove_grouppagepermission_permission_type"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="revision",
            name="submitted_for_moderation",
        ),
    ]



class MenuItem:
    """
    Describes a simple menu item in the Wagtail admin interface, which may be
    used in different contexts, e.g. inside dropdowns or as a standalone item.
    This class does not define how it should be rendered; that is up to the menu
    renderer to use the description provided here.
    """

    def __init__(self, label: str, url: str, icon_name="", priority=1000):
        self.label = label
        self.url = url
        self.icon_name = icon_name
        self.priority = priority

    def is_shown(self, user):
        """
        Whether this menu item should be shown for the given user; permission
        checks etc. should go here. By default, menu items are shown all the time.
        """
        return True

    def __lt__(self, other):
        if not isinstance(other, MenuItem):
            return NotImplemented
        return (self.priority, self.label) < (other.priority, other.label)



from django import forms
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.telepath import register
from wagtail.admin.widgets import BaseChooser, BaseChooserAdapter
from wagtail.images import get_image_model
from wagtail.images.shortcuts import get_rendition_or_not_found


class AdminImageChooser(BaseChooser):
    choose_one_text = _("Choose an image")
    choose_another_text = _("Change image")
    link_to_chosen_text = _("Edit this image")
    template_name = "wagtailimages/widgets/image_chooser.html"
    chooser_modal_url_name = "wagtailimages_chooser:choose"
    icon = "image"
    classname = "image-chooser"
    js_constructor = "ImageChooser"

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.model = get_image_model()

    def get_value_data_from_instance(self, instance):
        data = super().get_value_data_from_instance(instance)
        preview_image = get_rendition_or_not_found(instance, "max-165x165")
        data["preview"] = {
            "url": preview_image.url,
            "width": preview_image.width,
            "height": preview_image.height,
        }
        data["default_alt_text"] = instance.default_alt_text
        return data

    def get_context(self, name, value_data, attrs):
        context = super().get_context(name, value_data, attrs)
        context["preview"] = value_data.get("preview", {})
        context["default_alt_text"] = value_data.get("default_alt_text", "")
        return context

    @property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("wagtailimages/js/image-chooser-modal.js"),
                versioned_static("wagtailimages/js/image-chooser.js"),
                versioned_static("wagtailimages/js/image-chooser-telepath.js"),
            ]
        )


class ImageChooserAdapter(BaseChooserAdapter):
    js_constructor = "wagtail.images.widgets.ImageChooser"

    @cached_property
    def media(self):
        return forms.Media(
            js=[
                versioned_static("wagtailimages/js/image-chooser-modal.js"),
                versioned_static("wagtailimages/js/image-chooser-telepath.js"),
            ]
        )


register(ImageChooserAdapter(), AdminImageChooser)



# Generated by Django 3.2.3 on 2021-05-25 13:26

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="EmailUser",
            fields=[
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "uuid",
                    models.UUIDField(
                        default=uuid.uuid4, primary_key=True, serialize=False
                    ),
                ),
                ("email", models.EmailField(max_length=255, unique=True)),
                ("is_staff", models.BooleanField(default=True)),
                ("is_active", models.BooleanField(default=True)),
                ("first_name", models.CharField(blank=True, max_length=50)),
                ("last_name", models.CharField(blank=True, max_length=50)),
                ("is_superuser", models.BooleanField(default=False)),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.Permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from django.test import SimpleTestCase

from wagtail.test.utils.form_data import querydict_from_html


class TestQueryDictFromHTML(SimpleTestCase):
    html = """
    <form id="personal-details">
        <input type="hidden" name="csrfmiddlewaretoken" value="Z783HTL5Bc2J54WhAtEeR3eefM1FBkq0EbTfNnYnepFGuJSOfvosFvwjeKYtMwFr">
        <input type="hidden" name="no_value_input">
        <input type="hidden" value="no name input">
        <div>
            <div>
                <label>
                    <span>Full name</span>
                    <input type="text" name="name" value="Jane Doe" placeholder="">
                </label>
                <label>
                    <span>Email address</span>
                    <input type="email" name="email" value="jane@example.com" placeholder="name@example.com">
                </label>
            </div>
        </div>
    </form>
    <form id="event-details">
        <div>
            <div>
                <label>
                    <span>When is your event?</span>
                    <input type="date" name="date" value="2023-01-01">
                </label>
                <label>
                    <span>What type of event is it?</span>
                    <select name="event_type">
                        <option value="corporate">Corporate event</option>
                        <option value="wedding">Wedding</option>
                        <option value="birthday">Birthday</option>
                        <option value="other" selected>Other</option>
                    </select>
                </label>
                <label>
                    <span>What age groups is it suitable for?</span>
                    <select name="ages" multiple>
                        <option>Infants</option>
                        <option>Children</option>
                        <option>Teenagers</option>
                        <option selected>18-30</option>
                        <option selected>30-50</option>
                        <option>50-70</option>
                        <option>70+</option>
                    </select>
                </label>
            </div>
        </div>
    </form>
    <form id="market-research">
        <div>
            <div>
                <fieldset>
                    <legend>How many pets do you have?</legend>
                    <div class="radio-list">
                        <div class="radio">
                            <label>
                                <input type="radio" name="pets" value="0" />
                                None
                            </label>
                        </div>
                        <div class="radio">
                            <label>
                                <input type="radio" name="pets" value="1" />
                                One
                            </label>
                        </div>
                        <div class="radio">
                            <label>
                                <input type="radio" name="pets" value="2" checked />
                                Two
                            </label>
                        </div>
                        <div class="radio">
                            <label>
                                <input type="radio" name="pets" value="3+" />
                                Three or more
                            </label>
                        </div>
                    </div>
                </fieldset>
                <fieldset>
                    <legend>Which two colours do you like best?</legend>
                    <div class="checkbox-list">
                        <div class="checkbox">
                            <label>
                                <input type="checkbox" name="colours" value="cyan">
                                Cyan
                            </label>
                        </div>
                        <div class="checkbox">
                            <label>
                                <input type="checkbox" name="colours" value="magenta" checked />
                                Magenta
                            </label>
                        </div>
                        <div class="checkbox">
                            <label>
                                <input type="checkbox" name="colours" value="yellow" />
                                Yellow
                            </label>
                        </div>
                        <div class="checkbox">
                            <label>
                                <input type="checkbox" name="colours" value="black" checked />
                                Black
                            </label>
                        </div>
                        <div class="checkbox">
                            <label>
                                <input type="checkbox" name="colours" value="white" />
                                White
                            </label>
                        </div>
                    </div>
                </fieldset>
                <label>
                    <span>Tell us what you love</span>
                    <textarea name="love" rows="3">Comic books</textarea>
                </label>
            </div>
        </div>
    </form>
    """

    personal_details = [
        ("no_value_input", [""]),
        ("name", ["Jane Doe"]),
        ("email", ["jane@example.com"]),
    ]

    event_details = [
        ("date", ["2023-01-01"]),
        ("event_type", ["other"]),
        ("ages", ["18-30", "30-50"]),
    ]

    market_research = [
        ("pets", ["2"]),
        ("colours", ["magenta", "black"]),
        ("love", ["Comic books"]),
    ]

    def test_html_only(self):
        # data should be extracted from the 'first' form by default
        result = querydict_from_html(self.html)
        self.assertEqual(list(result.lists()), self.personal_details)

    def test_include_csrf(self):
        result = querydict_from_html(self.html, exclude_csrf=False)
        expected_result = [
            (
                "csrfmiddlewaretoken",
                ["Z783HTL5Bc2J54WhAtEeR3eefM1FBkq0EbTfNnYnepFGuJSOfvosFvwjeKYtMwFr"],
            )
        ] + self.personal_details
        self.assertEqual(list(result.lists()), expected_result)

    def test_form_index(self):
        for index, expected_data in (
            (0, self.personal_details),
            ("2", self.market_research),
            (1, self.event_details),
        ):
            result = querydict_from_html(self.html, form_index=index)
            self.assertEqual(list(result.lists()), expected_data)

    def test_form_id(self):
        for id, expected_data in (
            ("event-details", self.event_details),
            ("personal-details", self.personal_details),
            ("market-research", self.market_research),
        ):
            result = querydict_from_html(self.html, form_id=id)
            self.assertEqual(list(result.lists()), expected_data)

    def test_invalid_form_id(self):
        with self.assertRaises(ValueError):
            querydict_from_html(self.html, form_id="invalid-id")

    def test_invalid_index(self):
        with self.assertRaises(ValueError):
            querydict_from_html(self.html, form_index=5)



# Generated by Django 4.2.dev20221212103954 on 2022-12-12 11:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("tests", "0013_draftstatemodel_add_lockablemixin"),
    ]

    operations = [
        migrations.CreateModel(
            name="LockableModel",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "locked",
                    models.BooleanField(
                        default=False, editable=False, verbose_name="locked"
                    ),
                ),
                (
                    "locked_at",
                    models.DateTimeField(
                        editable=False, null=True, verbose_name="locked at"
                    ),
                ),
                ("text", models.TextField()),
                (
                    "locked_by",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="locked_%(class)ss",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="locked by",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



import re
from types import SimpleNamespace
from urllib.parse import parse_qs, urlsplit, urlunsplit

from django.conf import settings
from django.utils.http import url_has_allowed_host_and_scheme, urlencode
from django.utils.translation import gettext


def get_admin_base_url():
    """
    Gets the base URL for the wagtail admin site. This is set in `settings.WAGTAILADMIN_BASE_URL`.
    """
    return getattr(settings, "WAGTAILADMIN_BASE_URL", None)


def get_valid_next_url_from_request(request):
    next_url = request.POST.get("next") or request.GET.get("next")
    if not next_url or not url_has_allowed_host_and_scheme(
        url=next_url, allowed_hosts={request.get_host()}
    ):
        return ""
    return next_url


def get_latest_str(obj):
    """
    Helper function to get the latest string representation of an object.
    Draft changes are saved as revisions instead of immediately reflected to the
    instance, so this function utilises the latest revision's object_str
    attribute if available.
    """
    from wagtail.models import DraftStateMixin, Page

    if isinstance(obj, Page):
        result = obj.specific_deferred.get_admin_display_title()
    elif isinstance(obj, DraftStateMixin) and obj.latest_revision:
        result = obj.latest_revision.object_str
    else:
        result = str(obj)

    if result.strip() == "":
        result = gettext("%(classname)s object (%(id)s)") % {
            "classname": obj.__class__.__name__,
            "id": obj.pk,
        }

    return result


def get_user_display_name(user):
    """
    Returns the preferred display name for the given user object: the result of
    user.get_full_name() if implemented and non-empty, or user.get_username() otherwise.
    """
    try:
        full_name = user.get_full_name().strip()
        if full_name:
            return full_name
    except AttributeError:
        pass

    try:
        return user.get_username()
    except AttributeError:
        # we were passed None or something else that isn't a valid user object; return
        # empty string to replicate the behaviour of {{ user.get_full_name|default:user.get_username }}
        return ""


def set_query_params(url: str, params: dict):
    """
    Given a URL and a dictionary of query parameters,
    returns a new URL with those query parameters added or updated.

    If the value of a query parameter is None, that parameter will be removed from the URL.
    """

    scheme, netloc, path, query, fragment = urlsplit(url)
    querydict = parse_qs(query)
    querydict.update(params)
    querydict = {key: value for key, value in querydict.items() if value is not None}
    query = urlencode(querydict, doseq=True)
    return urlunsplit((scheme, netloc, path, query, fragment))


def get_keyboard_key_labels_from_request(request):
    """
    Returns an instance of SimpleNamespace based on the user's keyboard layout
    based on the User-Agent header in the request.

    These are intentionally not translated, as they are key labels that are assumed
    to be consistent across all languages.
    """

    user_agent = request.headers.get("User-Agent", "")
    is_mac_os = re.search(r"Mac|iPod|iPhone|iPad", user_agent)

    labels = {
        "ALT": "⌥" if is_mac_os else "Alt",
        "CMD": "⌘" if is_mac_os else "Ctrl",
        "CTRL": "^" if is_mac_os else "Ctrl",
        "DEL": "Delete",
        "ENTER": "Return" if is_mac_os else "Enter",
        "ESC": "Esc",
        "MOD": "⌘" if is_mac_os else "Ctrl",
        "SHIFT": "Shift",
        "TAB": "Tab",
    }

    return SimpleNamespace(**labels)



from django.test import TestCase
from django.urls import reverse

from wagtail.models import Page
from wagtail.test.testapp.models import (
    FormPageWithRedirect,
    PageChooserModel,
    SimplePage,
)
from wagtail.test.utils import WagtailTestUtils
from wagtail.test.utils.template_tests import AdminTemplateTestUtils


class TestPageUsage(AdminTemplateTestUtils, WagtailTestUtils, TestCase):
    # We don't show the "Home" breadcrumb item in page views
    base_breadcrumb_items = []

    def setUp(self):
        self.user = self.login()
        self.root_page = Page.objects.get(id=2)

        with self.captureOnCommitCallbacks(execute=True):
            page = SimplePage(
                title="Hello world!",
                slug="hello-world",
                content="hello",
            )
            self.root_page.add_child(instance=page)
            page.save_revision().publish()
        self.page = SimplePage.objects.get(id=page.id)

    def test_simple(self):
        usage_url = reverse("wagtailadmin_pages:usage", args=(self.page.id,))
        response = self.client.get(usage_url)

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing.html")
        self.assertContains(response, "Usage")
        self.assertContains(response, "Hello world!")

        items = [
            {
                "url": reverse("wagtailadmin_explore_root"),
                "label": "Root",
            },
            {
                "url": reverse("wagtailadmin_explore", args=(self.root_page.id,)),
                "label": "Welcome to your new Wagtail site!",
            },
            {
                "url": reverse("wagtailadmin_explore", args=(self.page.id,)),
                "label": "Hello world! (simple page)",
            },
            {
                "url": "",
                "label": "Usage",
                "sublabel": "Hello world! (simple page)",
            },
        ]
        self.assertBreadcrumbsItemsRendered(items, response.content)

        # There should be exactly one edit link, rendered as a header button
        edit_url = reverse("wagtailadmin_pages:edit", args=(self.page.id,))
        soup = self.get_soup(response.content)
        edit_links = soup.select(f"a[href='{edit_url}']")
        self.assertEqual(len(edit_links), 1)
        edit_link = edit_links[0]
        classes = edit_link.attrs.get("class")
        self.assertIn("w-header-button", classes)
        self.assertIn("button", classes)

    def test_has_private_usage(self):
        with self.captureOnCommitCallbacks(execute=True):
            PageChooserModel.objects.create(page=self.page)
        usage_url = reverse("wagtailadmin_pages:usage", args=(self.page.id,))
        response = self.client.get(usage_url)

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing.html")
        self.assertContains(response, "Usage")
        self.assertContains(response, "Hello world!")

        self.assertContains(response, "(Private page chooser model)")
        self.assertContains(response, "<td>Page chooser model</td>", html=True)

    def test_has_editable_usage(self):
        with self.captureOnCommitCallbacks(execute=True):
            form_page = FormPageWithRedirect(
                title="Contact us",
                slug="contact-us",
                to_address="to@email.com",
                from_address="from@email.com",
                subject="The subject",
                thank_you_redirect_page=self.page,
            )

            form_page = self.root_page.add_child(instance=form_page)

        usage_url = reverse("wagtailadmin_pages:usage", args=(self.page.id,))
        response = self.client.get(usage_url)

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing.html")
        self.assertContains(response, "Usage")
        self.assertContains(response, "Hello world!")

        self.assertContains(response, "Contact us")
        self.assertContains(
            response,
            reverse("wagtailadmin_pages:edit", args=(form_page.id,))
            + "#:w:contentpath=thank_you_redirect_page",
        )
        self.assertContains(response, "Thank you redirect page")
        self.assertContains(response, "<td>Form page with redirect</td>", html=True)

    def test_pagination(self):
        with self.captureOnCommitCallbacks(execute=True):
            for _ in range(50):
                PageChooserModel.objects.create(page=self.page)

        usage_url = reverse("wagtailadmin_pages:usage", args=(self.page.id,))
        response = self.client.get(f"{usage_url}?p=2")

        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, "wagtailadmin/generic/listing.html")
        self.assertContains(response, "Page 2 of 3")
        self.assertContains(response, f"{usage_url}?p=1")
        self.assertContains(response, f"{usage_url}?p=2")
        self.assertContains(response, f"{usage_url}?p=3")



# Generated by Django 5.1.3 on 2025-04-21 10:43

from django.db import migrations, models

import wagtail.models.preview


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0054_delete_notyetregisteredgenericsetting_and_more"),
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.CreateModel(
            name="PreviewableGenericSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.models.preview.PreviewableMixin, models.Model),
        ),
        migrations.CreateModel(
            name="PreviewableSiteSetting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.TextField()),
                (
                    "site",
                    models.OneToOneField(
                        editable=False,
                        on_delete=models.deletion.CASCADE,
                        to="wagtailcore.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(wagtail.models.preview.PreviewableMixin, models.Model),
        ),
    ]



from django.core.exceptions import PermissionDenied

from wagtail.log_actions import log


class DeletePagePermissionError(PermissionDenied):
    """
    Raised when the page delete cannot be performed due to insufficient permissions.
    """

    pass


class DeletePageAction:
    def __init__(self, page, user):
        self.page = page
        self.user = user

    def check(self, skip_permission_checks=False):
        if (
            self.user
            and not skip_permission_checks
            and not self.page.permissions_for_user(self.user).can_delete()
        ):
            raise DeletePagePermissionError(
                "You do not have permission to delete this page"
            )

    def _delete_page(self, page, *args, **kwargs):
        from wagtail.models import Page

        # Ensure that deletion always happens on an instance of Page, not a specific subclass. This
        # works around a bug in treebeard <= 3.0 where calling SpecificPage.delete() fails to delete
        # child pages that are not instances of SpecificPage
        if type(page) is Page:
            for child in page.get_descendants().specific().iterator():
                self.log_deletion(child)
            self.log_deletion(page.specific)

            # this is a Page instance, so carry on as we were
            return super(Page, page).delete(*args, **kwargs)
        else:
            # retrieve an actual Page instance and delete that instead of page
            return DeletePageAction(
                Page.objects.get(id=page.id), user=self.user
            ).execute(*args, **kwargs)

    def execute(self, *args, skip_permission_checks=False, **kwargs):
        self.check(skip_permission_checks=skip_permission_checks)

        return self._delete_page(self.page, *args, **kwargs)

    def log_deletion(self, page):
        log(
            instance=page,
            action="wagtail.delete",
            user=self.user,
            deleted=True,
        )



from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from wagtail.models import Site
from wagtail.permission_policies.sites import SitePermissionPolicy
from wagtail.test.testapp.models import ImportantPagesSiteSetting, TestSiteSetting
from wagtail.test.utils import WagtailTestUtils
from wagtail.tests.permission_policies.test_permission_policies import (
    PermissionPolicyTestUtils,
)


class PermissionPolicyTestCase(PermissionPolicyTestUtils, WagtailTestUtils, TestCase):
    def setUp(self):
        super().setUp()
        self.policy = SitePermissionPolicy(TestSiteSetting)

        self.default_site = Site.objects.get(is_default_site=True)
        self.other_site = Site.objects.create(
            hostname="other.example.com",
            port=80,
            root_page=self.default_site.root_page,
            is_default_site=False,
        )

        self.content_type = ContentType.objects.get_for_model(TestSiteSetting)
        self.change_permission = Permission.objects.get(
            content_type=self.content_type, codename="change_testsitesetting"
        )

        # groups
        self.site_owners = Group.objects.create(name="Site owners")
        self.site_owners.permissions.add(self.change_permission)

        self.default_site_owners = Group.objects.create(name="Default site owners")
        self.default_site_owners.site_permissions.create(
            permission=self.change_permission, site=self.default_site
        )
        self.other_site_owners = Group.objects.create(name="Other site owners")
        self.other_site_owners.site_permissions.create(
            permission=self.change_permission, site=self.other_site
        )

        self.superuser = self.create_superuser(
            "superuser", "superuser@example.com", "password"
        )
        self.inactive_superuser = self.create_superuser(
            "inactivesuperuser", "inactivesuperuser@example.com", "password"
        )
        self.inactive_superuser.is_active = False
        self.inactive_superuser.save()

        # users
        self.site_owner = self.create_user(
            "siteowner", "siteowner@example.com", "password"
        )
        self.site_owner.groups.add(self.site_owners)

        self.direct_site_owner = self.create_user(
            "directsiteowner", "directsiteowner@example.com", "password"
        )
        self.direct_site_owner.user_permissions.add(self.change_permission)

        self.default_site_owner = self.create_user(
            "defaultsiteowner", "defaultsiteowner@example.com", "password"
        )
        self.default_site_owner.groups.add(self.default_site_owners)

        self.other_site_owner = self.create_user(
            "othersiteowner", "othersiteowner@example.com", "password"
        )
        self.other_site_owner.groups.add(self.other_site_owners)


class TestSiteSettingPermissionPolicy(PermissionPolicyTestCase):
    def test_user_has_permission(self):
        self.assertUserPermissionMatrix(
            [
                (self.superuser, True),
                (self.inactive_superuser, False),
                (self.site_owner, True),
                (self.direct_site_owner, True),
                (self.default_site_owner, True),
                (self.other_site_owner, True),
            ],
            actions=["change"],
        )

    def test_user_has_permission_for_site(self):
        self.assertUserInstancePermissionMatrix(
            self.default_site,
            [
                (self.superuser, True),
                (self.inactive_superuser, False),
                (self.site_owner, True),
                (self.direct_site_owner, True),
                (self.default_site_owner, True),
                (self.other_site_owner, False),
            ],
            actions=["change"],
        )

    def test_user_has_permission_for_site_setting(self):
        site_setting = TestSiteSetting.objects.create(
            site=self.default_site,
            title="Default site",
            email="defaultsite@example.com",
        )
        self.assertUserInstancePermissionMatrix(
            site_setting,
            [
                (self.superuser, True),
                (self.inactive_superuser, False),
                (self.site_owner, True),
                (self.direct_site_owner, True),
                (self.default_site_owner, True),
                (self.other_site_owner, False),
            ],
            actions=["change"],
        )

    def test_users_with_permission(self):
        self.assertResultSetEqual(
            self.policy.users_with_permission("change"),
            [
                self.superuser,
                self.site_owner,
                self.direct_site_owner,
                self.default_site_owner,
                self.other_site_owner,
            ],
        )

    def test_users_with_permission_for_site(self):
        self.assertResultSetEqual(
            self.policy.users_with_permission_for_instance("change", self.default_site),
            [
                self.superuser,
                self.site_owner,
                self.direct_site_owner,
                self.default_site_owner,
            ],
        )

    def test_users_with_permission_for_setting(self):
        site_setting = TestSiteSetting.objects.create(
            site=self.default_site,
            title="Default site",
            email="defaultsite@example.com",
        )
        self.assertResultSetEqual(
            self.policy.users_with_permission_for_instance("change", site_setting),
            [
                self.superuser,
                self.site_owner,
                self.direct_site_owner,
                self.default_site_owner,
            ],
        )

    def test_sites_user_has_permission_for(self):
        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(self.superuser, "change"),
            [self.default_site, self.other_site],
        )

        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(
                self.inactive_superuser, "change"
            ),
            [],
        )

        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(self.site_owner, "change"),
            [self.default_site, self.other_site],
        )

        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(self.direct_site_owner, "change"),
            [self.default_site, self.other_site],
        )

        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(
                self.default_site_owner, "change"
            ),
            [self.default_site],
        )

        self.assertResultSetEqual(
            self.policy.sites_user_has_permission_for(self.other_site_owner, "change"),
            [self.other_site],
        )

    def test_instances_user_has_permission_for(self):
        site_setting = TestSiteSetting.objects.create(
            site=self.default_site,
            title="Default site",
            email="defaultsite@example.com",
        )
        # other_site does not have a TestSiteSetting instance, so will be omitted from results

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(self.superuser, "change"),
            [site_setting],
        )

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(
                self.inactive_superuser, "change"
            ),
            [],
        )

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(self.site_owner, "change"),
            [site_setting],
        )

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(
                self.direct_site_owner, "change"
            ),
            [site_setting],
        )

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(
                self.default_site_owner, "change"
            ),
            [site_setting],
        )

        self.assertResultSetEqual(
            self.policy.instances_user_has_permission_for(
                self.other_site_owner, "change"
            ),
            [],
        )


class TestOtherModelPolicy(PermissionPolicyTestCase):
    """Permissions assigned on TestSiteSetting should not 'leak' to other models."""

    def setUp(self):
        super().setUp()
        self.policy = SitePermissionPolicy(ImportantPagesSiteSetting)

    def test_user_has_permission(self):
        self.assertUserPermissionMatrix(
            [
                (self.superuser, True),
                (self.inactive_superuser, False),
                (self.site_owner, False),
                (self.direct_site_owner, False),
                (self.default_site_owner, False),
                (self.other_site_owner, False),
            ],
            actions=["change"],
        )

    def test_user_has_permission_for_site(self):
        self.assertUserInstancePermissionMatrix(
            self.default_site,
            [
                (self.superuser, True),
                (self.inactive_superuser, False),
                (self.site_owner, False),
                (self.direct_site_owner, False),
                (self.default_site_owner, False),
                (self.other_site_owner, False),
            ],
            actions=["change"],
        )

    def test_users_with_permission_for_site(self):
        self.assertResultSetEqual(
            self.policy.users_with_permission_for_instance("change", self.default_site),
            [
                self.superuser,
            ],
        )



from django.urls import path

from wagtail.images.views import images, multiple

app_name = "wagtailimages"
urlpatterns = [
    path("", images.IndexView.as_view(), name="index"),
    path("results/", images.IndexView.as_view(results_only=True), name="index_results"),
    path("<int:image_id>/", images.EditView.as_view(), name="edit"),
    path("<int:image_id>/delete/", images.DeleteView.as_view(), name="delete"),
    path(
        "<int:image_id>/generate_url/",
        images.URLGeneratorView.as_view(),
        name="url_generator",
    ),
    path(
        "<int:image_id>/generate_url/<str:filter_spec>/",
        images.GenerateURLView.as_view(),
        name="generate_url",
    ),
    path("<int:image_id>/preview/<str:filter_spec>/", images.preview, name="preview"),
    path("add/", images.CreateView.as_view(), name="add"),
    path("usage/<int:image_id>/", images.UsageView.as_view(), name="image_usage"),
    path("multiple/add/", multiple.AddView.as_view(), name="add_multiple"),
    path("multiple/<int:image_id>/", multiple.EditView.as_view(), name="edit_multiple"),
    path(
        "multiple/create_from_uploaded_image/<int:uploaded_file_id>/",
        multiple.CreateFromUploadedImageView.as_view(),
        name="create_multiple_from_uploaded_image",
    ),
    path(
        "multiple/<int:image_id>/delete/",
        multiple.DeleteView.as_view(),
        name="delete_multiple",
    ),
    path(
        "multiple/delete_upload/<int:uploaded_file_id>/",
        multiple.DeleteUploadView.as_view(),
        name="delete_upload_multiple",
    ),
]



from django.template import Context, Template
from django.test import RequestFactory, TestCase
from django.utils.html import format_html

from wagtail.admin.ui.tables import (
    BaseColumn,
    Column,
    RelatedObjectsColumn,
    Table,
    TitleColumn,
)
from wagtail.models import Page, Site


class TestTable(TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.rf = RequestFactory()

    def render_component(self, obj):
        request = self.rf.get("/")
        template = Template("{% load wagtailadmin_tags %}{% component obj %}")
        return template.render(Context({"request": request, "obj": obj}))

    def test_table_render(self):
        data = [
            {"first_name": "Paul", "last_name": "Simon"},
            {"first_name": "Art", "last_name": "Garfunkel"},
        ]

        table = Table(
            [
                Column("first_name"),
                Column("last_name"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <thead>
                    <tr><th>First name</th><th>Last name</th></tr>
                </thead>
                <tbody>
                    <tr><td>Paul</td><td>Simon</td></tr>
                    <tr><td>Art</td><td>Garfunkel</td></tr>
                </tbody>
            </table>
        """,
        )

    def test_table_render_with_caption(self):
        data = [
            {"first_name": "Paul", "last_name": "Simon"},
            {"first_name": "Art", "last_name": "Garfunkel"},
        ]

        caption = "Test table"

        table = Table(
            columns=[
                Column("first_name"),
                Column("last_name"),
            ],
            data=data,
            caption=caption,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <caption class="w-sr-only">Test table</caption>
                <thead>
                    <tr><th>First name</th><th>Last name</th></tr>
                </thead>
                <tbody>
                    <tr><td>Paul</td><td>Simon</td></tr>
                    <tr><td>Art</td><td>Garfunkel</td></tr>
                </tbody>
            </table>
        """,
        )

    def test_table_render_with_width(self):
        data = [
            {"first_name": "Paul", "last_name": "Simon"},
            {"first_name": "Art", "last_name": "Garfunkel"},
        ]

        table = Table(
            [
                Column("first_name"),
                Column("last_name", width="75%"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <col />
                <col width="75%" />
                <thead>
                    <tr><th>First name</th><th>Last name</th></tr>
                </thead>
                <tbody>
                    <tr><td>Paul</td><td>Simon</td></tr>
                    <tr><td>Art</td><td>Garfunkel</td></tr>
                </tbody>
            </table>
        """,
        )

    def test_title_column(self):
        root_page = Page.objects.filter(depth=2).first()
        blog = Site.objects.create(
            hostname="blog.example.com", site_name="My blog", root_page=root_page
        )
        gallery = Site.objects.create(
            hostname="gallery.example.com", site_name="My gallery", root_page=root_page
        )
        data = [blog, gallery]

        table = Table(
            [
                TitleColumn(
                    "hostname",
                    url_name="wagtailsites:edit",
                    link_classname="choose-site",
                    link_attrs={"data-chooser": "yes"},
                ),
                Column("site_name", label="Site name"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <thead>
                    <tr><th>Hostname</th><th>Site name</th></tr>
                </thead>
                <tbody>
                    <tr>
                        <td class="title">
                            <div class="title-wrapper">
                                <a href="/admin/sites/edit/%d/" class="choose-site" data-chooser="yes">blog.example.com</a>
                            </div>
                        </td>
                        <td>My blog</td>
                    </tr>
                    <tr>
                        <td class="title">
                            <div class="title-wrapper">
                                <a href="/admin/sites/edit/%d/" class="choose-site" data-chooser="yes">gallery.example.com</a>
                            </div>
                        </td>
                        <td>My gallery</td>
                    </tr>
                </tbody>
            </table>
        """
            % (blog.pk, gallery.pk),
        )

    def test_column_media(self):
        class FancyColumn(Column):
            class Media:
                js = ["js/gradient-fill.js"]

        data = [
            {"first_name": "Paul", "last_name": "Simon"},
            {"first_name": "Art", "last_name": "Garfunkel"},
        ]

        table = Table(
            [
                FancyColumn("first_name"),
                Column("last_name"),
            ],
            data,
        )

        self.assertIn('src="/static/js/gradient-fill.js"', str(table.media["js"]))

    def test_row_classname(self):
        class SiteTable(Table):
            def get_row_classname(self, instance):
                return "default-site" if instance.is_default_site else ""

        root_page = Page.objects.filter(depth=2).first()
        blog = Site.objects.create(
            hostname="blog.example.com",
            site_name="My blog",
            root_page=root_page,
            is_default_site=True,
        )
        gallery = Site.objects.create(
            hostname="gallery.example.com", site_name="My gallery", root_page=root_page
        )
        data = [blog, gallery]

        table = SiteTable(
            [
                Column("hostname"),
                Column("site_name", label="Site name"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <thead>
                    <tr><th>Hostname</th><th>Site name</th></tr>
                </thead>
                <tbody>
                    <tr class="default-site">
                        <td>blog.example.com</td>
                        <td>My blog</td>
                    </tr>
                    <tr>
                        <td>gallery.example.com</td>
                        <td>My gallery</td>
                    </tr>
                </tbody>
            </table>
        """,
        )

    def test_row_attrs(self):
        class SiteTable(Table):
            def get_row_attrs(self, instance):
                attrs = super().get_row_attrs(instance)
                attrs["data-id"] = instance.pk
                return attrs

        root_page = Page.objects.filter(depth=2).first()
        blog = Site.objects.create(
            hostname="blog.example.com",
            site_name="My blog",
            root_page=root_page,
            is_default_site=True,
        )
        gallery = Site.objects.create(
            hostname="gallery.example.com", site_name="My gallery", root_page=root_page
        )
        data = [blog, gallery]

        table = SiteTable(
            [
                Column("hostname"),
                Column("site_name", label="Site name"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            f"""
            <table class="listing">
                <thead>
                    <tr><th>Hostname</th><th>Site name</th></tr>
                </thead>
                <tbody>
                    <tr data-id="{blog.pk}">
                        <td>blog.example.com</td>
                        <td>My blog</td>
                    </tr>
                    <tr data-id="{gallery.pk}">
                        <td>gallery.example.com</td>
                        <td>My gallery</td>
                    </tr>
                </tbody>
            </table>
        """,
        )

    def test_table_and_row_in_context(self):
        data = [
            {"first_name": "Paul", "last_name": "Simon"},
            {"first_name": "Art", "last_name": "Garfunkel"},
        ]

        class CounterColumn(BaseColumn):
            def render_cell_html(self, instance, parent_context):
                context = self.get_cell_context_data(instance, parent_context)
                return format_html(
                    "<td>{} of {}</td>",
                    context["row"].index + 1,
                    context["table"].row_count,
                )

        table = Table(
            [
                CounterColumn("index"),
                Column("first_name"),
                Column("last_name"),
            ],
            data,
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <thead>
                    <tr><th>Index</th><th>First name</th><th>Last name</th></tr>
                </thead>
                <tbody>
                    <tr><td>1 of 2</td><td>Paul</td><td>Simon</td></tr>
                    <tr><td>2 of 2</td><td>Art</td><td>Garfunkel</td></tr>
                </tbody>
            </table>
        """,
        )


class TestRelatedObjectsColumn(TestCase):
    def setUp(self):
        self.rf = RequestFactory()

    def render_component(self, obj):
        request = self.rf.get("/")
        template = Template("{% load wagtailadmin_tags %}{% component obj %}")
        return template.render(Context({"request": request, "obj": obj}))

    def test_table_render(self):
        table = Table(
            [
                Column("title"),
                RelatedObjectsColumn("sites_rooted_here"),
            ],
            Page.objects.all(),
        )

        html = self.render_component(table)
        self.assertHTMLEqual(
            html,
            """
            <table class="listing">
                <thead>
                    <tr><th>Title</th><th>Sites rooted here</th></tr>
                </thead>
                <tbody>
                    <tr>
                        <td>Root</td>
                        <td></td>
                    </tr>
                    <tr>
                        <td>Welcome to your new Wagtail site!</td>
                        <td><ul><li>localhost [default]</li></ul></td>
                    </tr>
                </tbody>
            </table>
        """,
        )
